from b2bTools.general.Io import B2bIo


class AlignmentsIO:

	@staticmethod
	def read_alignments(fileName, fileType=None, resetAlignRefSeqID=False, gapCode='-'):
		"""
		Reads an Alignment file detecting the file format

		:param fileName: Path to the file
		:return: List of (seqId, sequenceString) tuples
		"""

		return B2bIo().readAlignments(fileName, fileType=fileType, resetAlignRefSeqID=resetAlignRefSeqID, gapCode=gapCode)

	@staticmethod
	def read_alignments_fasta(fileName, resetAlignRefSeqID=False, gapCode='-'):
		"""
		Reads an Alignment file in FASTA format

		:param fileName: Path to the FASTA file
		:return: List of (seqId, sequenceString) tuples
		"""

		return B2bIo().readAlignments(fileName, fileType="FASTA", resetAlignRefSeqID=resetAlignRefSeqID, gapCode=gapCode)

	@staticmethod
	def read_alignments_A3M(fileName, resetAlignRefSeqID=False, gapCode='-'):
		"""
		Reads an Alignment file in A3M format

		:param fileName: Path to the FASTA file
		:return: List of (seqId, sequenceString) tuples
		"""

		return B2bIo().readAlignments(fileName, fileType="A3M", resetAlignRefSeqID=resetAlignRefSeqID, gapCode=gapCode)

	@staticmethod
	def read_alignments_blast(fileName, resetAlignRefSeqID=False, gapCode='-'):
		"""
		Reads an Alignment file in FASTA format

		:param fileName: Path to the FASTA file
		:return: List of (seqId, sequenceString) tuples
		"""

		return B2bIo().readAlignments(fileName, fileType="Blast", resetAlignRefSeqID=resetAlignRefSeqID, gapCode=gapCode)

	@staticmethod
	def read_alignments_balibase(fileName, resetAlignRefSeqID=False, gapCode='-'):
		return B2bIo().readAlignments(fileName, fileType="BaliBase", resetAlignRefSeqID=resetAlignRefSeqID, gapCode=gapCode)

	@staticmethod
	def read_alignments_clustal(fileName, resetAlignRefSeqID=False, gapCode='-'):
		return B2bIo().readAlignments(fileName, fileType="CLUSTAL", resetAlignRefSeqID=resetAlignRefSeqID, gapCode=gapCode)

	@staticmethod
	def read_alignments_psi(fileName, resetAlignRefSeqID=False, gapCode='-'):
		return B2bIo().readAlignments(fileName, fileType="PSI", resetAlignRefSeqID=resetAlignRefSeqID, gapCode=gapCode)

	@staticmethod
	def read_alignments_phylip(fileName, resetAlignRefSeqID=False, gapCode='-'):
		return B2bIo().readAlignments(fileName, fileType="PHYLIP", resetAlignRefSeqID=resetAlignRefSeqID, gapCode=gapCode)

	@staticmethod
	def read_alignments_stockholm(fileName, resetAlignRefSeqID=False, gapCode='-'):
		return B2bIo().readAlignments(fileName, fileType="STOCKHOLM", resetAlignRefSeqID=resetAlignRefSeqID, gapCode=gapCode)

	@staticmethod
	def write_fasta_from_alignment(alignFile, outFastaFile):
		return B2bIo().writeFastaFromAlignment(alignFile, outFastaFile)

	@staticmethod
	def write_fasta_from_seq_alignment_dict(seqAlignments, outFastaFile):
		return B2bIo().writeFastaFromSeqAlignmentDict(seqAlignments, outFastaFile)

	@staticmethod
	def json_preds_to_csv_singleseq(json_data, output_dir='results'):
		"""
		Function to save the predictions in the JSON file to a txt-like file
		:param json_data: Input data in JSON format
		:output_dir: Optional path to the output directory
		"""

		return B2bIo().json_preds_to_csv_singleseq(json_data, output_dir)

	@staticmethod
	def json_preds_to_csv_msa(json_data, file_prefix, output_dir='results'):
		"""
		Function to save the predictions in the CSV file to a txt-like file
		:param json_data: Input data in JSON format
		:output_dir: Optional path to the output directory
		:param file_prefix: The prefix of the output file (<output_dir>/<prefix>_b2btools_preds.txt)
		"""

		return B2bIo().json_preds_to_csv_msa(json_data, id=file_prefix, output_dir=output_dir)



"""
======================COPYRIGHT/LICENSE START==========================

peopleAndCitationsIO.py: I/O for NMR-STAR citation and people information

Copyright (C) 2008 Wim Vranken (European Bioinformatics Institute)

=======================================================================

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

A copy of this license can be found in ../../../../license/LGPL.license

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA


======================COPYRIGHT/LICENSE END============================

for further information, please contact :

- CCPN website (http://www.ccpn.ac.uk/)
- PDBe website (http://www.ebi.ac.uk/pdbe/)

- contact Wim Vranken (wim@ebi.ac.uk)
=======================================================================

If you are using this software for academic purposes, we suggest
quoting the following references:

===========================REFERENCE START=============================
R. Fogh, J. Ionides, E. Ulrich, W. Boucher, W. Vranken, J.P. Linge, M.
Habeck, W. Rieping, T.N. Bhat, J. Westbrook, K. Henrick, G. Gilliland,
H. Berman, J. Thornton, M. Nilges, J. Markley and E. Laue (2002). The
CCPN project: An interim report on a data model for the NMR community
(Progress report). Nature Struct. Biol. 9, 416-418.

Wim F. Vranken, Wayne Boucher, Tim J. Stevens, Rasmus
H. Fogh, Anne Pajon, Miguel Llinas, Eldon L. Ulrich, John L. Markley, John
Ionides and Ernest D. Laue (2005). The CCPN Data Model for NMR Spectroscopy:
Development of a Software Pipeline. Proteins 59, 687 - 696.

===========================REFERENCE END===============================
"""

# Import general functions
#from b2bTools.general.ccpn.universal.Util import returnInt

from b2bTools.general.ccpn.format.general.formatIO import Person, Citation, Keyword

from b2bTools.general.ccpn.format.nmrStar.generalIO import NmrStarFile, NmrStarGenericFile

#####################
# Class definitions #
#####################

# Can not be read in separately, only as part of projectIO

class NmrStarPeopleAndCitationsFile(NmrStarGenericFile):

  """
  Information on file level.
  """

  def initialize(self,parent,saveFrame,infoType):

    self.persons = []
    self.personInGroups = []
    self.citations = []
    self.keywords = []

    self.saveFrame = saveFrame
    self.infoType = infoType

    self.parent = parent
    self.version = parent.version

    if self.saveFrame:
      self.parseSaveFrame()

  def parseSaveFrame(self):

    if not self.checkVersion():
      return

    if self.version in ('3.0','3.1'):

      personTagNames = ['Given_name','Middle_initials','Family_name','Family_title','Name_salutation']

      #
      # Entry level information
      #

      if self.saveFrame.name == 'entry_information':

        if self.infoType == 'contactPersons':

          personInGroupTags = ['Email_address', 'Department_and_institution', 'Mailing_address', 'Address_1',
                               'Address_2', 'Address_3', 'City', 'State_province', 'Country', 'Postal_code',
                               'Phone_number', 'FAX_number', 'Role', 'Organization_type']

          #
          # Contact person information
          #

          tableName = '_Contact_person'
          if tableName in self.saveFrame.tables:

            tableTags = self.saveFrame.tables[tableName].tags

            for i in range(0,len(tableTags['ID'])):

              personInfo = []
              for personTagName in personTagNames:
                personInfo.append(tableTags[personTagName][i])

              self.persons.append(Person(self,serial = tableTags['ID'][i],*personInfo))

              personInGroupInfo = []
              for persInGrpTag in personInGroupTags:
                personInGroupInfo.append(tableTags[persInGrpTag][i])

              self.personInGroups.append(PersonInGroup(self,serial = tableTags['ID'][i],*personInGroupInfo))


        elif self.infoType == 'authors':

          #
          # Entry author information
          #

          tableName = '_Entry_author'
          if tableName in self.saveFrame.tables:
            tableTags = self.saveFrame.tables[tableName].tags

            for i in range(0,len(tableTags['Ordinal'])):

              personInfo = []
              for personTagName in personTagNames[:-1]:
                personInfo.append(tableTags[personTagName][i])

              self.persons.append(Person(self,serial = tableTags['Ordinal'][i],*personInfo))

      #
      # Citations
      #

      elif self.saveFrame.name == 'citations':

        #
        # Create citation information...
        #

        citationType = self.saveFrame.tags['Type']

        if self.saveFrame.tags['Class'] ==  "entry citation":
          self.isPrimary = True
        else:
          self.isPrimary = False

        citationTagNames = ['Type','Title','Status']

        if citationType == 'journal':
          citationTagNames += ['Journal_abbrev','Journal_volume','Journal_issue','Page_first','Year',None,None,None,'isPrimary','Page_last','Journal_name_full','PubMed_ID','Details',None,None,'MEDLINE_UI_code','DOI']
        elif citationType == 'book':
          citationTagNames += [None,'Book_volume',None,'Page_first','Year','Book_publisher_city','Book_publisher',None,'isPrimary','Page_last',None,'PubMed_ID','Details','Book_title','Book_series','MEDLINE_UI_code','DOI']
        elif citationType == 'conference' or citationType == 'abstract':
          citationTagNames += [None,None,None,'Page_first','Year','Conference_site',None,'Conference_country','isPrimary','Page_last',None,'PubMed_ID','Details','Conference_title',None,'MEDLINE_UI_code','DOI']
        elif citationType == 'thesis':
          citationTagNames += [None,None,None,'Page_first','Year','Thesis_institution_city','Thesis_institution','Thesis_institution_country','isPrimary','Page_last',None,'PubMed_ID','Details',None,None,'MEDLINE_UI_code','DOI']
        elif citationType == 'BMRB only':
          citationTagNames += [None,None,None,'Page_first','Year',None,None,None,'isPrimary','Page_last',None,'PubMed_ID','Details',None,None,'MEDLINE_UI_code','DOI']

        # TODO
        #'Journal_ASTM': [None,lambda x = value: returnStarLine(x,length = 127),None,False],
        #'Journal_ISSN': [None,lambda x = value: returnStarLine(x,length = 127),None,False],
        #'Journal_CSD': [None,lambda x = value: returnStarLine(x,length = 127),None,False],
        #'Book_chapter_title': [None,lambda x = value: returnStarString(x,length = 255),None,False],
        #'Book_ISBN': [None,lambda x = value: returnStarLine(x,length = 127),None,False],
        #'Conference_state_province': [None,lambda x = value: returnStarLine(x,length = 127),None,False],
        #'Conference_start_date': [None,returnStarDateTime,None,False],
        #'Conference_end_date': [None,returnStarDateTime,None,False],
        #'Conference_abstract_number': [None,lambda x = value: returnStarLine(x,length = 127),None,False],
        #'WWW_URL': [None,returnStarString,None,False],

        citationInfo = [self]
        for tagName in citationTagNames:
          value = None
          if tagName in ('isPrimary',):
            value = getattr(self,tagName)
          elif tagName in self.saveFrame.tags:
            value = self.saveFrame.tags[tagName]
          citationInfo.append(value)

        self.citations.append(Citation(*citationInfo))

        #
        # Citation author information
        #

        tableName = '_Citation_author'
        if tableName in self.saveFrame.tables:
          tableTags = self.saveFrame.tables[tableName].tags

          for i in range(0,len(tableTags['Ordinal'])):

            personInfo = []
            for personTagName in personTagNames[:-1]:
              personInfo.append(tableTags[personTagName][i])

            self.persons.append(Person(self,serial = tableTags['Ordinal'][i],*personInfo))

            self.citations[-1].setAuthor(self.persons[-1])

        tableName = '_Citation_editor'
        if tableName in self.saveFrame.tables:
          tableTags = self.saveFrame.tables[tableName].tags

          for i in range(0,len(tableTags['Ordinal'])):

            personInfo = []
            for personTagName in personTagNames[:-1]:
              personInfo.append(tableTags[personTagName][i])

            self.persons.append(Person(self,serial = tableTags['Ordinal'][i],*personInfo))

            self.citations[-1].setEditor(self.persons[-1])

        tableName = '_Citation_keyword'

        tagNames = ('Keyword',)

        if tableName in self.saveFrame.tables:
          tableTags = self.saveFrame.tables[tableName].tags

          for i in range(0,len(tableTags['Keyword'])):

            infoList = []
            for tagName in tagNames:
              infoList.append(tableTags[tagName][i])

            self.keywords.append(Keyword(self,*infoList))

            self.citations[-1].setKeyword(self.keywords[-1])

    elif self.version == '2.1.1':
      pass

class PersonInGroup:

  def __init__(self, parent, emailAddr, dept, mailAddr, addr1, addr2, addr3, city, stateProv, country, postCode, phoneNo, faxNo, role, orgType, serial = None):

    self.parent = parent

    self.emailAddr = emailAddr
    self.dept = dept
    self.mailAddr = mailAddr
    self.addr1 = addr1
    self.addr2 = addr2
    self.addr3 = addr3
    self.city = city
    self.stateProv = stateProv
    self.country = country
    self.postCode = postCode
    self.phoneNo = phoneNo
    self.faxNo = faxNo
    self.role = role
    self.orgType = orgType

    self.serial = serial



TOOL_DYNAMINE = 'dynamine'
TOOL_DISOMINE = 'disomine'
TOOL_EFOLDMINE = 'efoldmine'
TOOL_AGMATA = 'agmata'
TOOL_PSP = 'psp'

DEPENDENCIES = {
    TOOL_DYNAMINE: [],
    TOOL_DISOMINE: [TOOL_DYNAMINE, TOOL_EFOLDMINE],
    TOOL_EFOLDMINE: [TOOL_DYNAMINE],
    TOOL_AGMATA: [TOOL_DYNAMINE, TOOL_EFOLDMINE],
    TOOL_PSP: []
}


"""
======================COPYRIGHT/LICENSE START==========================

Util.py: Useful functions for scripts in this directory and its subdirectories

Copyright (C) 2005-2008 Wim Vranken (European Bioinformatics Institute)

=======================================================================

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.
 
A copy of this license can be found in ../../../../license/LGPL.license
 
This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.
 
You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA


======================COPYRIGHT/LICENSE END============================

for further information, please contact :

- CCPN website (http://www.ccpn.ac.uk/)
- PDBe website (http://www.ebi.ac.uk/pdbe/)

- contact Wim Vranken (wim@ebi.ac.uk)
=======================================================================

If you are using this software for academic purposes, we suggest
quoting the following references:

===========================REFERENCE START=============================
R. Fogh, J. Ionides, E. Ulrich, W. Boucher, W. Vranken, J.P. Linge, M.
Habeck, W. Rieping, T.N. Bhat, J. Westbrook, K. Henrick, G. Gilliland,
H. Berman, J. Thornton, M. Nilges, J. Markley and E. Laue (2002). The
CCPN project: An interim report on a data model for the NMR community
(Progress report). Nature Struct. Biol. 9, 416-418.

Wim F. Vranken, Wayne Boucher, Tim J. Stevens, Rasmus
H. Fogh, Anne Pajon, Miguel Llinas, Eldon L. Ulrich, John L. Markley, John
Ionides and Ernest D. Laue (2005). The CCPN Data Model for NMR Spectroscopy:
Development of a Software Pipeline. Proteins 59, 687 - 696.

===========================REFERENCE END===============================
"""

import re
from b2bTools.general.ccpn.format.general.Constants import defaultSeqInsertCode

# characters that can be used as a one-digit index
indexChars = '123456789'

#######################
# Regular expressions #
#######################

def getRegularExpressions(format = None):

  patt = {}
  
  patt['emptyline'] = re.compile("^\s*$")
  patt['hash'] = re.compile("^\s*\#")
  patt['exclamation'] = re.compile("^\s*\!")
  patt['underscore'] = re.compile("^\s*\_")
  patt['colon'] = re.compile(":")
  patt['onlyDollar'] = re.compile("^\$$")
  patt['onlydigit'] = re.compile("^\d+$")
  patt['onlyFloat'] = re.compile("^\d+\.\d+$")
  patt['bracketEither'] = re.compile("\(|\)")
  patt['bracketOpen'] = re.compile("\s*\(")
  patt['bracketClose'] = re.compile("\)")
  patt['nucleusLetterDigit'] = re.compile("^([A-Za-z])(\d+)")
  patt['seqInsertCode'] = re.compile("^([A-Za-z]*)(\d+)([A-Za-z]*)")
  patt['anySpace'] = re.compile("\s")
  
  if format == 'amber':

    patt[format + 'RestraintStart'] = re.compile("(\&rst)")
    patt[format + 'RestraintEnd'] = re.compile("(\&end)")
  
  elif format == 'ansig':

    patt[format + 'CurlyBrace'] = re.compile("\{(.*)\}")

  elif format == 'auremol':

    patt[format + 'Section'] = re.compile("^\s*section_(.+)$")
    patt[format + 'SubSection'] = re.compile("^\s*([A-Z_]+)\:")

  elif format == 'autoAssign':
    patt[format + 'specPhase'] = re.compile("phase:\s*\{(.*)\}\s*\{\s*$")
    patt[format + 'onlyCurlyBraceEnd'] = re.compile("^\}$")   
    patt[format + 'CurlyBrace'] = re.compile("\{\s*(.*)\s*\}")
    patt[format + 'seqCode1Or3LetterAndCode'] = re.compile("([A-Z][a-z]{0,2})(\d+)")
    patt[format + 'spinSystemInfo'] = re.compile(".+(\(.+\))")

  elif format == 'aqua':
    patt[format + 'Count'] = re.compile("^count (\d+) type (.+)$")   
    patt[format + 'UpperDistance'] = re.compile("^NOEUPP")
    patt[format + 'LowerDistance'] = re.compile("^NOELOW")
    patt[format + 'EndComment']    = re.compile("\#(.+)$")

  elif format == 'bruker':
    patt[format + 'StartDoubleHash'] = re.compile("^\#\#")
    patt[format + 'StartDoubleHashDollar'] = re.compile("^\#\#\$")
    patt[format + 'StartDoubleDollar'] = re.compile("^\$\$")
    patt[format + 'SharpBracketOpen'] = re.compile("^\<")
    patt[format + 'SharpBracketClose'] = re.compile("\>$")
    patt[format + 'SharpBracketEither'] = re.compile("\<|\>")
    patt[format + 'BracketsListIndicator'] = re.compile("\(\d+\.\.\d+\)")
    patt[format + 'Date'] = re.compile("2\d\d\d-\d\d-\d\d")
    patt[format + 'DigitDotDigit'] = re.compile("^\-?\d*\.?\d+$")
    patt[format + 'Dot'] = re.compile("\.")
    patt[format + 'BracketCloseNumber'] = re.compile("\)\s*\d")
    patt[format + 'FinalReturn'] = re.compile("\n$")
    patt[format + 'BracketMultiValue'] = re.compile("\((\d+)\-(\d+)\)")
    patt[format + 'InOrDecrement'] = re.compile("\s*(i|d)d(\d+)")
    patt[format + 'HashComment'] = re.compile("^\s*\#(.+)")
    patt[format + 'SemiColonComment'] = re.compile("^\s*\;(.*)")
    patt[format + 'PulseName'] = re.compile("\/([^\/]+)\"")
    patt[format + 'FnMode'] = re.compile("FnMode?\s*(.+)\s*", re.I)
    patt[format + 'EchoAntiEcho'] = re.compile("igrad|\*EA", re.I)
    patt[format + 'PathwayElements'] = re.compile("(F\d+)\s*\(([^\)]+)\)")
    patt[format + 'DimIncrement'] = re.compile("in(\d+)\s*\=\s*inf(\d+)\s*\\s*/\s*(\d+)")
    patt[format + 'phaseSensitive'] = re.compile("\;\s*phase sensitive")
    patt[format + 'constantTime'] = re.compile("constant time")
    patt[format + 'semiconstantTime'] = re.compile("semi-constant time")
    patt[format + 'tSearch'] = re.compile("(t\d)")
    patt[format + 'Version'] = re.compile("Version\s*(\d+\.*\d*)")

  elif format == 'charmm':

    patt[format + 'NumberAtoms'] = re.compile("^\s*(\d+)\s*$")
    patt[format + 'AtomLine'] = re.compile("^\s*\d+\s+\d+\s+[A-Za-z]+\s+")

  # This is for CNS/xplor
  elif format == 'cns':
  
    patt[format + 'DistancePeakInfoLine'] = re.compile("peak",re.IGNORECASE)
    patt[format + 'MultiSign'] = re.compile("[\%\#\*\+]")
    patt[format + 'RestrNum'] = re.compile("\{\s*(\-?\d+)\s*\}")
    patt[format + 'Assign'] = re.compile("assi",re.IGNORECASE)
    patt[format + 'AssignOr'] = re.compile("or",re.IGNORECASE)
    patt[format + 'ChemShiftFormat'] = re.compile("attr\s+store",re.IGNORECASE)
    patt[format + 'ChemShiftStore'] = re.compile("do.+store\d+\s*\=\s*(\-?\d*\.?\d*)\s*\)",re.IGNORECASE)
    patt[format + 'ChainCode'] = re.compile("segid\s+(\"([A-Za-z0-9 ]+)\"|([A-Za-z0-9]+))\s*",re.IGNORECASE)
    patt[format + 'Class'] = re.compile("class\s+",re.IGNORECASE)
    patt[format + 'SeqCode'] = re.compile("resid?u?e?\s+(\-?\d+[a-zA-Z]?)\s*",re.IGNORECASE)
    patt[format + 'AtomName'] = re.compile("name\s+([A-Z0-9\%\#\*\+]+\'?)",re.IGNORECASE)
    patt[format + 'RestrDistances'] = re.compile("([\d\.]+)\s+(-?[\d\.]+)\s+(-?[\d\.?]+)")
    patt[format + 'RestrAngles'] = re.compile("(\d+\.?\d*)\s+(-?\+?\d+\.?\d*)\s+(\+?\d+\.?\d*)\s+(\d)")
    patt[format + 'RestrCoupling'] = re.compile("(\d+\.?\d*)\s+(\d+\.?\d*)")
    patt[format + 'RestrCsa'] = re.compile("(-?\d+\.?\d*)\s+(\d+\.?\d*)\s*(\d+\.?\d*)?")
    patt[format + 'RestrRdc'] = re.compile("(-?\d+\.?\d*)\s+(\d+\.?\d*)\s*(\d+\.?\d*)?")
    patt[format + 'RestrInnerOr'] = re.compile("\s*or",re.IGNORECASE)
    patt[format + 'InnerElementPatt'] = re.compile("\(([^\)\(]+)\)",re.M)
    patt[format + 'LongCommentStart'] = re.compile("\{")
    patt[format + 'LongCommentEnd'] = re.compile("\}")

  elif format == 'csi':
  
    patt[format + 'Comment'] = re.compile("^\s*\>")

  elif format in ['dyana','cyana']:
  
    patt[format + 'CoordinateInfoLine'] = re.compile("\s*\d+\s+[A-Z0-9]+\s+\d+\s+")
    patt[format + 'CoordinateAtomLine'] = re.compile("^ATOM|^HETATM")
    patt[format + 'NewModel'] = re.compile("^MODEL\s+(\d+)")

  elif format == 'mars':
    
    patt[format + 'AtomNameHeader'] = re.compile("^\s*([\sNCHABO\-1])+\s*$")
    patt[format + 'AtomInfo'] = re.compile("([A-Z]+)(\-\d)?")

  elif format == 'mol':

    patt[format + 'Counts'] = re.compile("^([0-9 ][0-9 ][0-9]){6}")
    patt[format + 'Atoms'] = re.compile("^([0-9\- ][0-9\- ][0-9\- ][0-9\- ][0-9]\.[0-9][0-9 ][0-9 ][0-9 ]){3}")
    patt[format + 'Bonds'] = re.compile("^([0-9 ][0-9 ][0-9]){4}([0-9 ][0-9 ][0-9 ])([0-9 ][0-9 ][0-9])")

  elif format == 'mol2':
    
    patt[format + 'TriposTag'] = re.compile("\@\<TRIPOS\>(.+)")

  elif format == 'monte':

    patt[format + 'Comment'] = re.compile("^\s*(\#\#|\%\%)")
    patt[format + 'Assignment'] = re.compile("([A-Za-z]+)?([0-9]+|\?)?")
    patt[format + 'AtomInfo'] = re.compile("([A-Za-z0-9]+)(\(.+\)|\-1)?")

  elif format in ['nmrDraw','talos']:
  
    patt[format + 'Remark'] = re.compile("^REMARK")
    patt[format + 'Dataline'] = re.compile("^DATA") 
    patt[format + 'Vars'] = re.compile("^VARS")
    patt[format + 'Format'] = re.compile("^FORMAT")
    
  elif format == 'nmrView':

    patt[format + 'DigitSpace'] = re.compile("^\d+\s+")
    patt[format + 'CurlyBrace'] = re.compile("\{([^{}]*)\}")
    patt[format + 'CurlyBraceStart'] = re.compile("\{")    
    patt[format + 'CurlyBraceEnd'] = re.compile("\}\s+")    
    patt[format + 'NumbersNoBrace'] = re.compile("^(\d+\.\d+\s*){2}")

  elif format == 'nmrStar':

    patt[format + 'EndSaveTag'] = re.compile("save_$")

  elif format == 'pdb':

    patt[format + 'NewModel'] = re.compile("^MODEL\s+(\d+)")
    patt[format + 'AllAtom'] = re.compile("^ATOM|^HETATM")
    patt[format + 'HetAtom'] = re.compile("^HETATM")
    patt[format + 'Header'] = re.compile("^HEADER")
    patt[format + 'Title'] = re.compile("^TITLE")
    patt[format + 'Remark4'] = re.compile("^REMARK   4 ([a-zA-Z0-9]{4}) COMPLIES WITH FORMAT V.\s*(\d+\.\d+)\s*\,")
    patt[format + 'Compound'] = re.compile("^COMPND\s+\d*\s+")
    patt[format + 'DbReference'] = re.compile("^DBREF")
    patt[format + 'SequenceChange'] = re.compile("^SEQADV")
    patt[format + 'Sequence'] = re.compile("^SEQRES")
    patt[format + 'Source'] = re.compile("^SOURCE\s+\d*\s+(.+)")
    patt[format + 'Keywds'] = re.compile("^KEYWDS\s+\d*\s+(.+)")
    patt[format + 'ExpData'] = re.compile("^EXPDATA\s+\d*\s+(.+)")
    patt[format + 'Authors'] = re.compile("^AUTHOR\s+\d*\s+(.+)")
    patt[format + 'Journal'] = re.compile("^JRNL\s+(AUTH|TITL|REF|REFN|PUBL|EDIT)\s+\d*\s+(.+)")
    patt[format + 'Reference'] = re.compile("^REFERENCE\s+(\d+)")
    patt[format + 'ReferenceJournal'] = re.compile("\s*(AUTH|TITL|REF|REFN|PUBL|EDIT)\s+\d*\s+(.+)")
    patt[format + 'Remarks'] = re.compile("^REMARK\s+(\d+)\s+(.+)")
    patt[format + 'HetGroup'] = re.compile("^HET\s+")
    patt[format + 'HetName'] = re.compile("^HETNAM")
    patt[format + 'HetSynonym'] = re.compile("^HETSYN")
    patt[format + 'HetFormula'] = re.compile("^FORMUL")
    patt[format + 'Bonds'] = re.compile("^CONECT")
    patt[format + 'SsBond'] = re.compile("^SSBOND")
    patt[format + 'Link'] = re.compile("^LINK")
    patt[format + 'SecStruc'] = re.compile("^(HELIX|TURN|SHEET)")
    patt[format + 'Termination'] = re.compile("^TER ")

  elif format == 'pipp':
    patt[format + '?_AXIS'] = re.compile("^(.+)_AXIS")
    patt[format + 'Shift'] = re.compile("^\s*(.+)\s+([0-9]*\.?[0-9]+)\s+\((.+)\)\s*$")
    patt[format + 'ShiftNoAss'] = re.compile("^\s*(.+)\s+([0-9]*\.?[0-9]+)\s*$")

  elif format == 'pistachio':
    patt[format + 'Comment'] = re.compile("^\s*\%.+")
  
  elif format == 'sparky':
    patt[format + 'SharpBracketBetween'] = re.compile("^\<(.+)\>$")
    patt[format + 'LabelCodeName'] = re.compile("([A-Z]?)((\d+\,?)+)(.+)")
    patt[format + 'BracketBetween'] = re.compile("\((.+)\)")

  # Addition Maxim Mayzel, Gothenburg
  elif format == 'targetedAcquisition':
    patt[format + 'seqCode1Or1LetterAndCode'] = re.compile("([A-Z])(\d+)")
    #patt[format + 'spinSystemInfo'] = re.compile(".+(\(.+\))")
    patt[format + 'figOfMerit'] = re.compile("([B|G|M])\s*(\d+\.*\d*)")

  elif format == 'xeasy':
    patt[format + 'IName'] = re.compile("^\#\s*INAME\s+(\d)\s*(.+)$")
    patt[format + 'CyanaFormat'] = re.compile("^\#\s*CYANAFORMAT\s+(.+)\s*$")
    patt[format + 'PeakInfo'] = re.compile("^\s*([^#]+)\s*\#?.*$")


  """
  
  #
  # TODO: WHERE ARE THESE USED!!?!
  #
  patt['infoDirs'] = re.compile("^info\..+")
  patt['projectDirs'] = re.compile("^project\..+")
  patt['varFiles'] = re.compile("^vars\..+")
  patt['nmrStarFiles'] = re.compile("^bmr(\d+)\.str")
   # patt[format + 'DMXTypes'] = re.compile("^DMX|DRX|Avance$")
   # patt[format + 'AllTypes'] = re.compile("^DMX|DRX|Avance|AM|AMX$")

  # This is for nmrPipe
  patt['var2pipe'] = re.compile("var2pipe$")
  patt['bruk2pipe'] = re.compile("bruk2pipe$")
  patt['pipe'] = re.compile("\|")
  patt['backslash'] = re.compile('\\\\')
  patt['dashstart'] = re.compile("^\-")
  patt['x'] = re.compile("^x")
  patt['y'] = re.compile("^y")
  patt['z'] = re.compile("^z")
  patt['a'] = re.compile("^a[^(q2d)]")

  # This is for dataNavigator
  patt['pipe2Dproc'] = re.compile("^test\.ft$")
  patt['pipe3Dproc'] = re.compile("^test(\d+)\.ft$")

  # These are for nmrPipe and Azara exec... scripts.
  patt['pipeConv'] = re.compile("^autoconv\.([HCN]+)\.com$")
  patt['pipeProc'] = re.compile("^autoproc\.([HCN]+)\.com$")

  # These are for DataManager
  patt['Varian'] = re.compile("procpar$")
  patt['Bruker'] = re.compile("acqu\d?s?$")

  # For converters, exportNmrProcPars
  patt['sensitivityEnhanced'] = re.compile("(echoAntiEcho|ranceKay) dim (\d)")
  """
  return patt

patt = getRegularExpressions()

def getSeqAndInsertCode(seqCode):

  seqInsertCode = defaultSeqInsertCode
  
  if seqCode != None:
    try:
      seqCode = int(seqCode)
    except:
      searchObj = patt['seqInsertCode'].search(str(seqCode))
      if searchObj:
        try:
          seqCode = int(searchObj.group(2))       
          if searchObj.group(1) or searchObj.group(3):
            seqInsertCode = searchObj.group(1) + searchObj.group(3)

        except:
          seqCode = None
      else:
        seqCode = None
  
  return (seqCode,seqInsertCode)
  
def standardNucleusName(name):

  if (not name):
    name = '1H'
  elif ((name[0].upper() == 'H') or (name.upper() == '1H')):
    name = '1H'
  elif ((name[0].upper() == 'C') or (name.upper() == '13C')):
    name = '13C'
  elif ((name[0].upper() == 'N') or (name.upper() == '15N')):
    name = '15N'
  elif ((name[0].upper() == 'P') or (name.upper() == '31P')):
    name = '31P'
  else:
    name = '1H'

  return name




#!/usr/bin/env python
# encoding: utf-8
"""
Predictor.py

Created by Wim Vranken on 1st May 2020, as wrapper around original Disomine code
Note that the original DisoMine code was upgraded to Python3.7 and simplified by taking out redundant code for production version
The DisoMine code was also modified to better work in conjunction with the data structures used here.
Copyright (c) 2020- Wim Vranken. All rights reserved.
"""

from b2bTools.general.Io import B2bIo
from b2bTools.nmr.shiftCrypt.shiftCrypt import Standalone

class ShiftCrypt(B2bIo):

  version = '1.0'
  scriptName = "b2bTools.nmr.shiftCrypt.Predictor"
  name = "ShiftCrypt"

  def __init__(self):
    # Additional info for writing files
    self.references = ['doi: 10.1038/s41467-019-10322-w', 'doi: 10.1093/nar/gkaa391']
    self.infoTexts = ['Generated by ShiftCrypt', 'See http://bio2byte.be']

    self.informationPerPredictor = {
      self.name: {
        'references': self.references,
        'info':       ";".join(self.infoTexts),
        'version':    self.version,
        'origin':     self.scriptName
      }
    }

  def predictShifts(self, proteinShifts, modelClass='2'):

    """
    :param proteinShifts: Shift information for a protein
    """

    shiftCrypt = Standalone()

    # TODO: Should here test which model is best suited!

    shiftCrypt.loadModel(modelClass, None)

    encryptedValues = shiftCrypt.encryptProteinShifts(proteinShifts)
    shiftCryptResults = shiftCrypt.organiseData(encryptedValues,proteinShifts)

    if not shiftCryptResults:
      print("ShiftCrypt failed....")

    return shiftCryptResults




"""
======================COPYRIGHT/LICENSE START==========================

formatIO.py: Generic superclass for format classes in subdirectories

Copyright (C) 2005-2009 Wim Vranken (European Bioinformatics Institute)

=======================================================================

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

A copy of this license can be found in ../../../../license/LGPL.license

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA


======================COPYRIGHT/LICENSE END============================

for further information, please contact :

- CCPN website (http://www.ccpn.ac.uk/)
- PDBe website (http://www.ebi.ac.uk/pdbe/)

- contact Wim Vranken (wim@ebi.ac.uk)
=======================================================================

If you are using this software for academic purposes, we suggest
quoting the following references:

===========================REFERENCE START=============================
R. Fogh, J. Ionides, E. Ulrich, W. Boucher, W. Vranken, J.P. Linge, M.
Habeck, W. Rieping, T.N. Bhat, J. Westbrook, K. Henrick, G. Gilliland,
H. Berman, J. Thornton, M. Nilges, J. Markley and E. Laue (2002). The
CCPN project: An interim report on a data model for the NMR community
(Progress report). Nature Struct. Biol. 9, 416-418.

Wim F. Vranken, Wayne Boucher, Tim J. Stevens, Rasmus
H. Fogh, Anne Pajon, Miguel Llinas, Eldon L. Ulrich, John L. Markley, John
Ionides and Ernest D. Laue (2005). The CCPN Data Model for NMR Spectroscopy:
Development of a Software Pipeline. Proteins 59, 687 - 696.

===========================REFERENCE END===============================
"""

import os

from b2bTools.general.ccpn.universal.Util import returnInt

from b2bTools.general.ccpn.format.general.Util import getRegularExpressions, getSeqAndInsertCode
from b2bTools.general.ccpn.format.general.Constants import defaultMolCode

class FileParseError:

  def __init__(self, value):

    self.value = value

  def __str__(self):

    if self.value.count("\n"):
      returnStr = self.value
    else:
      returnStr = repr(self.value)

    return returnStr

class FormatFileError(FileParseError):

  pass

class FormatFile:

  FileParseError = FileParseError
  FormatFileError = FormatFileError

  def __init__(self,name,*args,**keywds):

    self.name = name
    self.newline = os.linesep

    self.version = None
    if 'version' in keywds.keys():
      self.version = keywds['version']
      del(keywds['version'])

    self.setGeneric()
    self.patt = getRegularExpressions(self.format)

    self.initialize(*args,**keywds)

  def setGeneric(self):

    self.format = None

class Citation:

  def __init__(self,parent,type,title,status,pubShortName,volume,issue,firstPage,year,publisherPlace,publisher = None,country = None,isPrimary = False,lastPage = None, pubLongName = None, pubMedId = None, details = None, bookTitle = None, bookSeries = None, medline = None, doi = None):

    self.parent = parent

    self.type = type
    self.title = title
    self.status = status

    self.pubShortName = pubShortName
    self.pubLongName = pubLongName

    self.pubMedId = pubMedId
    self.medline = medline
    self.doi = doi

    self.publisherPlace = publisherPlace
    self.publisher = publisher
    self.country = country

    self.volume = volume
    self.issue = issue
    self.firstPage = firstPage
    self.lastPage = lastPage
    self.year = year

    self.details = details
    self.isPrimary = isPrimary

    self.bookTitle = bookTitle
    self.bookSeries = bookSeries

    self.authors = []
    self.editors = []
    self.keywords = []

  def setAuthor(self,person):
    self.authors.append(person)

  def setEditor(self,person):
    self.editors.append(person)

  def setKeyword(self,person):
    self.keywords.append(person)

class Person:

  def __init__(self,parent,firstName,initials,familyName,familyTitle = None, title = None, serial = None):
    self.parent = parent

    self.firstName = firstName
    self.initials = initials
    self.familyName = familyName
    self.familyTitle = familyTitle
    self.title = title

    self.serial = serial

class Keyword:

  def __init__(self,parent,keywd):

    self.parent = parent
    self.keywd = keywd

class Sequence:

  """
  Information on sequences stored in file
  """

  def __init__(self,molName = None, chainCode = defaultMolCode, *args, **keywds):

    if molName:
      self.molName = molName.strip()
    else:
      self.molName = molName

    self.chainCode = chainCode

    self.originalChainCode = chainCode

    self.elements = []

    self.isCircular = False

    self.secStrucInfo = {}

    self.setFormatSpecific(*args,**keywds)

  def setFormatSpecific(self,*args,**keywds):

    pass

class SequenceElement:

  """
  Information for an element in the sequence (residue)
  """

  def __init__(self,seqCode,*args,**keywds):

    (self.seqCode,self.seqInsertCode) = getSeqAndInsertCode(seqCode)

    self.formatCode = None
    self.atomNames = []
    self.hasCisPeptideBond = False

    self.bonds = {}

    self.setResidueCode(*args)

    self.setFormatSpecific(*args,**keywds)

    self.secStrucInfo = None

  def setResidueCode(self,*args):

    if len(args) >= 1:
      self.code3Letter = args[0].upper()

  def setFormatSpecific(self,*args,**keywds):

    pass

  def setFormatCode(self,formatCode):

     self.formatCode = formatCode

  def addAtomName(self,atomName):

    self.atomNames.append(atomName)

  def setCisPeptideBond(self):

    self.hasCisPeptideBond = True

  def setBond(self,bondType,atomName,bondedSeqEl,bondedAtomName):

    if not self.bonds.has_key(bondType):
      self.bonds[bondType] = {}

    if not self.bonds[bondType].has_key(atomName):
      self.bonds[bondType][atomName] = []

    if not (bondedSeqEl,bondedAtomName) in self.bonds[bondType][atomName]:
      self.bonds[bondType][atomName].append((bondedSeqEl,bondedAtomName))
      print ("  Set %s bond from %s.%s - %s.%s" % (bondType,self.seqCode,atomName,bondedSeqEl.seqCode,bondedAtomName))

  def setSecStrucInfo(self,secStrucType,secStrucSerial,specificInfo):

    self.secStrucInfo = (secStrucType,secStrucSerial,specificInfo)

class SpinSystem:

  """
  Information about a spin system
  """

  def __init__(self,spinSysCode,*args,**keywds):

    self.code = returnInt(spinSysCode)

    self.setFormatSpecific(*args,**keywds)

  def setFormatSpecific(self,*args,**keywds):

    pass

class GenericProcessingParsFile:

  """

  Generic processing parameters stuff

  """

  dimTypeFrequency = 'frequency'
  dimTypeFid = 'fid'
  dimTypeSampled = 'sampled'
  dimTypeNotProcessed= 'not processed'
  dimTypes = (dimTypeFrequency, dimTypeFid, dimTypeSampled, dimTypeNotProcessed)

  #
  # Processing parameters stuff - mapping between Wayne's stuff and the DataFormat stuff...
  #

  parNameMapping = {

      'ndim': ('numDim','Number of dimensions'),
      'file': ('inputFile','Name of input FID'),
      'head': ('headerSize','Header size in words'),
      'integer': ('integer','Data is integer (not floating point)'),
      'swap': ('swap','Swap data'),
      'big_endian': ('bigEndian','Data is big endian'),
      'varian': ('varian','Varian data'),
      'dim': ('dim','Dimension'),
      'npts': ('numPoints','Number of real points'),
      'block': ('blockSize','Block size'),
      'sw': ('spectralWidth','Spectral width'),
      'sf': ('spectrometerFreq','Spectrometer frequence'),
      'refppm': ('refPpm','Reference ppm'),
      'refpt': ('refPoint','Reference point'),
      'nuc': ('nucleus','Nucleus'),
      'dimType': ('dimType','Dimension type'),
      'sampledValues': ('sampledValues','Sampled values'),
  }

  def initGeneric(self):

    self.aPars = {}
    self.fPars = {}

    self.allParKeys = []
    self.parKeys = {}

    for parType in self.allPars.keys():
      parKeys = []
      for parInfo in self.allPars[parType]:
        parKeys.append(parInfo[0])
        if parType in ('mainPars','dimPar') and parInfo[1]:
          self.setValue(parInfo,parInfo[3],initialize = 1)

      self.allParKeys.extend(parKeys)
      self.parKeys[parType] = parKeys

  def initDims(self):

    if self.aPars.has_key('ndim') and self.aPars['ndim']:
      numDim = self.aPars['ndim']
    else:
      numDim = self.fPars['numDim']

    self.aPars['dimType'] = numDim * [self.dimTypeFrequency]

    for parInfo in self.allPars['dimPars']:

      self.setValue(parInfo,numDim * [None])

      for i in range(numDim):
        self.setListValue(parInfo,i,parInfo[3],initialize = 1)

  def setValue(self,parInfo,value,initialize = 0):

    if parInfo[1]:
      self.aPars[parInfo[1]] = value


    if not initialize and self.parNameMapping.has_key(parInfo[1]):
      if type(value) == type([]):
        value = value[:]
      self.fPars[self.parNameMapping[parInfo[1]][0]] = value

    return value

  def setListValue(self,parInfo,index,value, initialize = 0):


    if parInfo[1]:
      self.aPars[parInfo[1]][index] = value

    if not initialize and self.parNameMapping.has_key(parInfo[1]):
      # Reset to default
      # - necessary for AXNUC/NUC1 settings from processingParsIO.py
      if value is None and parInfo[-1]:
        value = parInfo[-1]

      self.fPars[self.parNameMapping[parInfo[1]][0]][index] = value

    return value

  def findParInfo(self, pars, key):

    for parInfo in pars:
      if parInfo[1] == key:
        return parInfo

    return None

#
# Generic chemcomps, only for use in DataFormat.py at the moment (10/2009). Wim
#

class GenericChemComp:

  def __init__(self,name,molType,atomNames):

    self.name = name
    self.molType = molType
    self.atomNames = atomNames

    self.atoms = []
    self.bonds = []

class GenericAtom:

  def __init__(self,serial,atomName):

    self.serial = serial
    self.name = atomName

    # Dirty determination of element symbol, could probably be done more consistently but not that relevant
    atomNameNoNumbers = ""

    for i in range(len(atomName)):

      if atomName[i] not in '0123456789':
        atomNameNoNumbers += atomName[i]

    #
    # Try to set this now...
    #

    elementSymbol = None

    if atomNameNoNumbers:
      if atomName[0] in 'CHNOPSF':
        elementSymbol = atomName[0]
      else:
        elementSymbol = atomName

    self.atomType = elementSymbol

    #self.otherCoords = []

  #def addInfo(self,atomInfo):
  #
  #  self.otherCoords.append({})
  #
  #  for attrName in ('x','y','z'):
  #    self.otherCoords[-1][attrName] = atomInfo[attrName]
"""
class GenericBond:

  def __init__(self,parent,atomSerial1,atomSerial2,bondType,bondStatus):

    self.parent = parent

    self.atomSerial1 = atomSerial1
    self.atomSerial2 = atomSerial2

    self.bondType = bondType

    self.bondStatus = bondStatus
"""



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  utils.py
#  
#  Copyright 2018 -ThinkPad-L540>
#  
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#  
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#  
#  
import os
import numpy as np
from sklearn.preprocessing import MinMaxScaler,StandardScaler
letters={'CYS': 'C', 'ASP': 'D', 'SER': 'S', 'GLN': 'Q', 'LYS': 'K', 'ASN': 'N', 'PRO': 'P', 'THR': 'T', 'PHE': 'F', 'ALA': 'A', 'HIS': 'H', 'GLY': 'G', 'ILE': 'I', 'LEU': 'L', 'ARG': 'R', 'TRP': 'W', 'VAL': 'V', 'GLU': 'E', 'TYR': 'Y', 'MET': 'M'}
def run_rsa(folder='datasets/rereferenced_nmr/',outdir='rsa_vals/'):
	rsabin='./pdbasa'
	#for i in os.listdir(folder):
	#	os.system(rsabin+' '+folder+i+' '+outdir+i)
	## parse ##
	diz={}
	v=[]
	seqs={}
	for i in os.listdir(outdir):
		nome=i.split('.')[0]
		diz[nome]=[]
		resold=1
		tmp=[]
		cont=0
		seq=''
		for j in open(outdir+i,'r').readlines():

			if j.strip()=='':
				continue
			if not 'ATOM' == j[0:4]:
				continue
			res=j[22:26]
			rsa=j[60:66]
			try:
				aa=letters[j[17:20]]
			except:
				continue
			v+=[float(rsa)]
			if res!=resold and cont!=0:
				
				diz[nome]+=[np.mean(tmp)]
				seq+=aa
				tmp=[]
			tmp+=[-float(rsa)]
			resold=res
			cont+=1

		diz[nome]+=[np.mean(tmp)]
		seq+=aa
		assert len(seq)==len(diz[nome])
		seqs[nome]=seq
	v=np.array(v).reshape((-1,1))
	scaler=MinMaxScaler()
	scaler.fit(v)


	for i in list(diz.keys()):
		diz[i]=scaler.transform(np.array(diz[i]).reshape((-1,1))).reshape((-1))
	return diz,seqs
def leggifasta(database): #legge un file fasta e lo converte in un dizionario
	f=open(database)
	uniprot=f.readlines()
	f.close()
	dizio={}
	for i in uniprot:

		if i[0]=='>':
				if '|' in i:
					
					uniprotid=i.strip('>\n').split('|')[1]
				else:
					uniprotid=i.strip('>\n')
				dizio[uniprotid]=''
		else:
			dizio[uniprotid]=dizio[uniprotid]+i.strip('\n')
	return dizio
def parse_uniprot_annotations(fil='dataset/all_seqs_dnabinding_uniprot.annotations'):
	diz={}
	salta=0
	for i in open(fil).readlines():
		a=i.split()
		try:
			bind_start=int(a[3])-1
			bind_end=int(a[4])
		except:
			salta+=1
			continue
		seq=a[1]
		p1=[0]*bind_start
		p2=[1]*(bind_end-bind_start)
		p3=[0]*(len(seq)-bind_end)
		p=p1+p2+p3
		assert len(p)==len(seq) 
		diz[a[0]]=(seq,p)
	#print 'got ',len(diz),'predictions. Saltate',salta
	return diz
def getScoresSVR(pred, real, threshold,FULL_SCORES=True):
	import math
	import numpy as np
	if len(pred) != len(real):
		raise Exception("ERROR: input vectors have differente len!")
	i = 0
	confusionMatrix = {}
	confusionMatrix["TP"] = confusionMatrix.get("TP", 0)
	confusionMatrix["FP"] = confusionMatrix.get("FP", 0)
	confusionMatrix["FN"] = confusionMatrix.get("FN", 0)
	confusionMatrix["TN"] = confusionMatrix.get("TN", 0)
	while i < len(real):
		if float(pred[i])<=threshold and (int(real[i])==0):
			confusionMatrix["TN"] = confusionMatrix.get("TN", 0) + 1
		if float(pred[i])<=threshold and int(real[i])==1:
			confusionMatrix["FN"] = confusionMatrix.get("FN", 0) + 1
		if float(pred[i])>=threshold and int(real[i])==1:
			confusionMatrix["TP"] = confusionMatrix.get("TP", 0) + 1
		if float(pred[i])>=threshold and int(real[i])==0:
			confusionMatrix["FP"] = confusionMatrix.get("FP", 0) + 1
		i += 1
	#print confusionMatrix
	if FULL_SCORES:
	
		#print "--------------------------------------------"
		#print confusionMatrix["TN"],confusionMatrix["FN"],confusionMatrix["TP"],confusionMatrix["FP"]
		#print "          | SSBOND            | FREE             |"
		#print "predBond  | TP: %d (%2.2f%%)  | FP: %d (%2.2f%%) |" % (confusionMatrix["TP"], (confusionMatrix["TP"]/float(confusionMatrix["TP"]+confusionMatrix["FN"]))*100, confusionMatrix["FP"], (confusionMatrix["FP"]/float(confusionMatrix["FP"]+confusionMatrix["TN"]))*100 )
		#print "predFree  | FN: %d (%2.2f%%)  | TN: %d (%2.2f%%) |" % (confusionMatrix["FN"],(confusionMatrix["FN"]/float(confusionMatrix["TP"]+confusionMatrix["FN"]))*100, confusionMatrix["TN"], (confusionMatrix["TN"]/float(confusionMatrix["FP"]+confusionMatrix["TN"]))*100)
		sen = (confusionMatrix["TP"]/float((confusionMatrix["TP"] + confusionMatrix["FN"])))
		spe = (confusionMatrix["TN"]/float((confusionMatrix["TN"] + confusionMatrix["FP"])))
		acc =  (confusionMatrix["TP"] + confusionMatrix["TN"])/float((sum(confusionMatrix.values())))
		bac = (0.5*((confusionMatrix["TP"]/float((confusionMatrix["TP"] + confusionMatrix["FN"])))+(confusionMatrix["TN"]/float((confusionMatrix["TN"] + confusionMatrix["FP"])))))
		inf =((confusionMatrix["TP"]/float((confusionMatrix["TP"] + confusionMatrix["FN"])))+(confusionMatrix["TN"]/float((confusionMatrix["TN"] + confusionMatrix["FN"])))-1.0)
		pre =(confusionMatrix["TP"]/float((confusionMatrix["TP"] + confusionMatrix["FP"])))
		mcc =	( ((confusionMatrix["TP"] * confusionMatrix["TN"])-(confusionMatrix["FN"] * confusionMatrix["FP"])) / math.sqrt((confusionMatrix["TP"]+confusionMatrix["FP"])*(confusionMatrix["TP"]+confusionMatrix["FN"])*(confusionMatrix["TN"]+confusionMatrix["FP"])*(confusionMatrix["TN"]+confusionMatrix["FN"])) )  
	
	#print real
	#print pred
	from sklearn.metrics import roc_auc_score
	aucScore = roc_auc_score(real, pred)
	
	#print "AUC = %3.3f" % aucScore
	#print "--------------------------------------------"
	if FULL_SCORES:
		return sen,spe,acc,pre,mcc,aucScore
	else:
		return aucScore
def run_ring(pdb_folder='pdb_interacting/'):
	############ you need to export VICTOR_ROOT=/home/gabriele/HD1/dna_binding_prediction/ring/ ####
	def parse_ring(fil,tmp_fold='tmp_ringfiles/'):
		fil=tmp_fold+fil
		diz={}
		for i in open(fil).readlines():
			a=i.split()
			if ':_:DA' in a[0] or ':_:DT' in a[0] or ':_:DC' in a[0] or ':_:DG' in a[0]:
				first_dna=True
			else:
				first_dna=False
			if ':_:DA' in a[2] or ':_:DT' in a[2] or ':_:DC' in a[2] or ':_:DG' in a[2]:
				second_dna=True
			else:
				second_dna=False
			if first_dna and not second_dna:
				resi=a[2].split(':')
				if resi[3] not in letters:
					print('MISSING AMINOACID')
					rname='X'
				else:
					rname=letters[resi[3]]
				if resi[0] not in diz:
					diz[resi[0]]={}
				diz[resi[0]][(int(resi[1]),rname)]=1
			if not first_dna and second_dna:
				resi=a[0].split(':')
				if resi[3] not in letters:
					print('MISSING AMINOACID')
					rname='X'
				else:
					rname=letters[resi[3]]
				if resi[0] not in diz:
					diz[resi[0]]={}
				diz[resi[0]][(int(resi[1]),rname)]=1
		return diz
	def parse_pdb(fil,tmp_fold='pdb_interacting/'):
		fil=tmp_fold+fil
		diz={}
		seq={}
		for i in open(fil).readlines():
			if i.strip()=='':
				continue
			a=i.split()
			if not a[0]=='ATOM':
				continue
			if 'DA' in a[3] or 'DT' in a[3] or 'DC' in a[3] or 'DG' in a[3]: #dna
				continue
			chain=i[21]#a[4]
			resnum=int(i[22:26])
			if chain not in seq:
				seq[chain]={}
			'''
			try:
				int(a[5])
			except:
				return 'error'
			'''
		
			if a[3] not in letters:
				print('MISSING AMINOACID')
				rname='X'
			else:
				rname=letters[a[3]]
			if (int(resnum),rname) not in seq[chain]:
						seq[chain][(int(resnum),rname)]=0
			#except:
			#	return 'error'
			'''
			except:
				i=i[:22]+' '+i[22:] ### ma porcodio...se i residui vanno sopra 999 il pdb si dimentica di aggiungere una colonna
				a=i.split()
				if not a[0]=='ATOM':
					continue
				if 'DA' in a[3] or 'DT' in a[3] or 'DC' in a[3] or 'DG' in a[3]: #dna
					continue
				chain=a[4]
				if not seq.has_key(chain):
					seq[chain]={}
				
				if not seq[chain].has_key((int(a[5]),letters[a[3]])):
					seq[chain][(int(a[5]),letters[a[3]])]=0
				#raw_input()
			'''
		return seq
	ring_bin='ring/bin/Ring'
	db={}
	cont=0
	for pdb in os.listdir(pdb_folder)[:]:
		if not '.pdb' in pdb:
			continue
		#if pdb.replace('.pdb','.edges') in os.listdir('tmp_ringfiles'):
		#	continue
		print('starting ',pdb)
		os.system(ring_bin+' -i '+pdb_folder+pdb+' -t 3 --all -E tmp_ringfiles/'+pdb.replace('.pdb','.edges')+' > /dev/null')

		diz=parse_ring(pdb.replace('.pdb','.edges'))
		pdb_diz=parse_pdb(pdb)
		if pdb_diz=='error':
			print('salto')
			continue
		### chain selection policy i take the one with the greatest number of interactions ###
		bestchain=0
		for i in list(diz.keys()):
			if len(diz[i])>bestchain:
				chain=i
				bestchain=len(diz[i])
		if bestchain==0:
			print('NO DNA INTERACTIONS IN',pdb.replace('.pdb',''))
			continue
		if chain not in pdb_diz:
			print('salto')
			continue
		for i in list(diz[chain].keys()):
			pdb_diz[chain][i]=1

		seq=''
		vals=[]
		for i in sorted(pdb_diz[chain].keys()):
			seq+=i[1]
			vals+=[pdb_diz[chain][i]]
		db[pdb.replace('.pdb','')]=(seq,vals)
		
			
	return db
def aucNth(yp, y, N=50):
	assert len(y) == len(yp)
	assert len(y) > 1
	from sklearn.metrics import roc_curve, auc, roc_auc_score 
	fpr, tpr, thresholds = roc_curve(y, yp)
	negatives = y.count(0)
	assert N < negatives
	perc = N / float(negatives)	
	#print perc
	fpr1k = []
	tpr1k = []
	i = 0
	while i < len(fpr):		
		if fpr[i] > perc:
			break
		fpr1k.append(fpr[i])
		tpr1k.append(tpr[i])
		i+=1	
	assert len(fpr1k) > 1
	#print fpr1k, tpr1k
	aucScore = auc(fpr1k, tpr1k) / perc 	
	#print ">>AUC%d: %3.3f" %(N,aucScore)
	return aucScore
def parse_official(fil='bound_free/bmr16065_bound.nef',bound=False):
	start=False
	arom_diz={  'F': ('CG','CD1','CD2','CE1','CE2','CZ'),
                'Y': ('CG','CD1','CD2','CE1','CE2','CZ'),
                'H': ('CD2','CE1','CG'),
                'W': ('CG','CD1','CD2','CE2','CE3','CH2','CZ2','CZ3')}

	oldnum=None
	chainold=None
	prot=protein_class()
	cominciato=False
	for l in open(fil).readlines():
		if not start:
			
			if '_Atom_shift_assign_ID' in l or '_nef_chemical_shift.value_uncertainty' in l:
				start=True
			else:
				prot.useless_stuf+=l
			continue

		if '_' in l:
			if not cominciato:
				prot.useless_stuf+=l+'\n'
			continue
		if 'loop_' in l:
			if not cominciato:
				prot.useless_stuf+=l+'\n'
			continue
		if 'stop_' in l:
			continue
		if 'save_' in l:
			continue	
		l=l.strip()
		if l=='':
			if not cominciato:
				prot.useless_stuf+='\n'
			continue
		
		cominciato=True
		a=l.replace('%','').split()
		chain=a[0]
		resitype=letters[a[2]]
		if resitype in arom_diz and a[3] in arom_diz[resitype]:
			atom=a[3]+'_'

		else:
			atom=a[3]

		shift=float(a[4])
		resinum=int(a[1])
		
		if oldnum==None:
			
			oldnum=resinum
			resi=residue_class()
			resi.resname=resitype
		if 	oldnum!=resinum:
			#print prot.seq,resi.resname
			prot.seq+=resi.resname
			prot.resi+=[resi]
			prot.resnumbering+=[resi.resinumbering]
			prot.chain+=[resi.chain]
			oldnum=resinum
			resi=residue_class()
		resi.atom[atom]=shift
		
		resi.resname=resitype
		resi.resname3letters=a[2]
		resi.chain=chain
		resi.resinumbering=resinum
		#def __init__(self):
		#	self.resname=None
		#	self.atom={}
		#	self.ss=None
		#	self.rci=None
		#	shift=float(a[4])
	return prot
if __name__ == '__main__':

	db=run_ring()
	print('ok')
	import pickle
	pickle.dump(db,open('marshalled/db_contacts_dna.m','w'))
	print(len(db))
	'''
	diz=leggifasta('dataset/cdhit_uniprot.fasta')
	diz1=parse_uniprot_annotations()
	buono=0
	for i in diz.keys():
		if diz1.has_key(i):
			buono+=1
	print buono
	'''



"""
======================COPYRIGHT/LICENSE START==========================

jCouplingIO.py: I/O for nmrStar J coupling saveframe

Copyright (C) 2008 Chris Penkett (European Bioinformatics Institute)

=======================================================================

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

A copy of this license can be found in ../../../../license/LGPL.license

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA


======================COPYRIGHT/LICENSE END============================

for further information, please contact :

- CCPN website (http://www.ccpn.ac.uk/)
- PDBe website (http://www.ebi.ac.uk/pdbe/)

- contact Chris Penkett (penkett@ebi.ac.uk)
=======================================================================

If you are using this software for academic purposes, we suggest
quoting the following references:

===========================REFERENCE START=============================
R. Fogh, J. Ionides, E. Ulrich, W. Boucher, W. Vranken, J.P. Linge, M.
Habeck, W. Rieping, T.N. Bhat, J. Westbrook, K. Henrick, G. Gilliland,
H. Berman, J. Thornton, M. Nilges, J. Markley and E. Laue (2002). The
CCPN project: An interim report on a data model for the NMR community
(Progress report). Nature Struct. Biol. 9, 416-418.

Wim F. Vranken, Wayne Boucher, Tim J. Stevens, Rasmus
H. Fogh, Anne Pajon, Miguel Llinas, Eldon L. Ulrich, John L. Markley, John
Ionides and Ernest D. Laue (2005). The CCPN Data Model for NMR Spectroscopy:
Development of a Software Pipeline. Proteins 59, 687 - 696.

===========================REFERENCE END===============================
"""

import os

from b2bTools.general.ccpn.universal.Io import getTopDirectory

from b2bTools.general.ccpn.format.nmrStar.generalIO import NmrStarGenericFile
from b2bTools.general.ccpn.format.nmrStar.generalIO import NmrStarFile

from b2bTools.general.ccpn.format.general.Util import getSeqAndInsertCode
from b2bTools.general.ccpn.format.general.Constants import defaultMolCode

#####################
# Class definitions #
#####################

class NmrStarFile(NmrStarFile):

  def initialize(self, version = '3.1'):

    self.jCouplingFiles = []

    self.files = self.jCouplingFiles

    if not self.version:
      self.version = version

    self.saveFrameName = 'coupling_constants'
    self.DataClassFile = NmrStarJCouplingFile

    self.components = [self.saveFrameName]
    self.setComponents()

  def read(self,verbose = 0):

    self.readComponent(verbose = verbose)


class NmrStarJCouplingFile(NmrStarGenericFile):

  """
  Information on file level
  """

  def initialize(self,parent,saveFrame = None):


    # Warning: should in principle be version specific, this is 3.1
    self.attrToTagMappings = (

        ('details','Details',None),
        ('specFreq','Spectrometer_frequency_1H',None),

    )

    self.jCouplingValues = []

    self.saveFrame = saveFrame

    self.parent = parent
    self.version = parent.version

    if self.saveFrame:
      self.parseSaveFrame()

  def parseSaveFrame(self):

    if not self.checkVersion():
      return

    #
    # Read the table with the measurements
    #

    tmpMolCode1 = tmpMolCode2 = None

    if self.version == '2.1.1':
      if self.saveFrame.tags.has_key('_Mol_system_component_name'):
        tmpMolCode1 = tmpMolCode2 = self.saveFrame.tags['_Mol_system_component_name']

      if '_Coupling_constant_code' not in self.saveFrame.tables.keys():
        return

      jCouplingTableTags = self.saveFrame.tables['_Coupling_constant_code'].tags
      numJCouplingValues = len(jCouplingTableTags['_Coupling_constant_code'])

    else:
      if self.attrToTagMappings:

        for (attrName,tagName,default) in self.attrToTagMappings:
          if self.saveFrame.tags.has_key(tagName):
            attrValue = self.saveFrame.tags[tagName]
          else:
            attrValue = default

          if not hasattr(self,attrName) or not self.attrName:
            setattr(self,attrName,attrValue)

      tableName = '_Coupling_constant'

      if self.saveFrame.tables.has_key(tableName):

        jCouplingTableTags = self.saveFrame.tables[tableName].tags
        numJCouplingValues = len(jCouplingTableTags['ID'])

    for i in range(numJCouplingValues):

      if not tmpMolCode1:
        if jCouplingTableTags.has_key('Entity_assembly_ID_1'):
          tmpMolCode1 = str(jCouplingTableTags['Entity_assembly_ID_1'][i])
        else:
          tmpMolCode1 = defaultMolCode

      if not tmpMolCode2:
        if jCouplingTableTags.has_key('Entity_assembly_ID_2'):
          tmpMolCode2 = str(jCouplingTableTags['Entity_assembly_ID_2'][i])
        else:
          tmpMolCode2 = defaultMolCode

      self.jCouplingValues.append(NmrStarJCoupling(tmpMolCode1,tmpMolCode2,self))
      self.jCouplingValues[-1].setData(jCouplingTableTags,i)

    tableName = '_Coupling_constant_experiment'
    self.setMeasureExperiments(tableName)

    tableName = '_Coupling_constant_software'
    self.setMeasureSoftwares(tableName)

class NmrStarJCoupling:

  def __init__(self,molCode1,molCode2,parent):

    self.molCode1 = molCode1
    self.molCode2 = molCode2
    self.parent = parent

  def setData(self,jCouplingTableTags,i):

    if self.parent.version == '2.1.1':
      assignList = [['type',      '_Coupling_constant_code',None],
                    ['seqCode1',  '_Atom_one_residue_seq_code',None],
                    ['resLabel1', '_Atom_one_residue_label',None],
                    ['atomName1', '_Atom_one_name',None],
                    ['seqCode2',  '_Atom_two_residue_seq_code',None],
                    ['resLabel2', '_Atom_two_residue_label',None],
                    ['atomName2', '_Atom_two_name',None],
                    ['value',     '_Coupling_constant_value',None],
                    ['valueMin',  '_Coupling_constant_min_value',None],
                    ['valueMax',  '_Coupling_constant_max_value',None],
                    ['valueError','_Coupling_constant_value_error',0.0]
                   ]
    else:
      assignList = [['Id',             'ID',None],
                    ['seqCode1',       'Seq_ID_1',None],
                    ['resLabel1',      'Comp_ID_1',None],
                    ['atomName1',      'Atom_ID_1',None],
                    ['atomType1',      'Atom_type_1',None],
                    ['seqCode2',       'Seq_ID_2',None],
                    ['resLabel2',      'Comp_ID_2',None],
                    ['atomName2',      'Atom_ID_2',None],
                    ['atomType2',      'Atom_type_2',None],
                    ['value',          'Val',None],
                    ['valueMin',       'Val_min',None],
                    ['valueMax',       'Val_max',None],
                    ['valueError',     'Val_err',0.0],
                    #['valueMin',       'Val_min',0.0],
                    #['valueMax',       'Val_max',0.0],
                    ['authorSeqCode1', 'Author_seq_ID_1',None],
                    ['authorSeqCode2', 'Author_seq_ID_2',None]
                    ]

    for (attrName,tagName,default) in assignList:

      if jCouplingTableTags.has_key(tagName):
        if jCouplingTableTags[tagName][i] != None:
          setattr(self,attrName,jCouplingTableTags[tagName][i])
        else:
          setattr(self,attrName,default)

    # For completeness...
    (self.seqCode1,self.seqInsertCode1) = getSeqAndInsertCode(self.seqCode1)
    (self.seqCode2,self.seqInsertCode2) = getSeqAndInsertCode(self.seqCode2)

###################
# Main of program #
###################

if __name__ == "__main__":

  files = ['/homes/penkett/project/nmrStar/files/new2/bmr15834_3.str']

  for file in files:

    #file = os.path.join(getTopDirectory(), file)

    nmrStarFile = NmrStarFile(file, version='3.1')

    nmrStarFile.read(verbose = 1)

    for jCouplingFile in nmrStarFile.jCouplingFiles:
      for jCoupling in jCouplingFile.jCouplingValues:
        print (jCoupling.Id, jCoupling.seqCode1, jCoupling.resLabel1, jCoupling.atomName1, jCoupling.seqCode2, jCoupling.resLabel2, jCoupling.atomName2, jCoupling.value, jCoupling.valueError)



"""Global constants


======================COPYRIGHT/LICENSE START==========================

Constants.py: Utility code for CCPN code generation framework

Copyright (C) 2005  (CCPN Project)

=======================================================================

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

A copy of this license can be found in ../../../license/LGPL.license

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA


======================COPYRIGHT/LICENSE END============================

for further information, please contact :

- CCPN website (http://www.ccpn.ac.uk/)

- email: ccpn@bioc.cam.ac.uk

=======================================================================

If you are using this software for academic purposes, we suggest
quoting the following references:

===========================REFERENCE START=============================
R. Fogh, J. Ionides, E. Ulrich, W. Boucher, W. Vranken, J.P. Linge, M.
Habeck, W. Rieping, T.N. Bhat, J. Westbrook, K. Henrick, G. Gilliland,
H. Berman, J. Thornton, M. Nilges, J. Markley and E. Laue (2002). The
CCPN project: An interim report on a data model for the NMR community
(Progress report). Nature Struct. Biol. 9, 416-418.

Rasmus H. Fogh, Wayne Boucher, Wim F. Vranken, Anne
Pajon, Tim J. Stevens, T.N. Bhat, John Westbrook, John M.C. Ionides and
Ernest D. Laue (2005). A framework for scientific data modeling and automated
software development. Bioinformatics 21, 1678-1684.

===========================REFERENCE END===============================
"""

# Special constants
True != 0
False != 1

trueString = 'True'
falseString = 'False'

dirsep = '/'



from b2bTools.general.Io import B2bIo


class NefIO:
    @staticmethod
    def read_nef_file(fileName):
        """
        :param fileName: Input NEF file path
        :return: File object with info, if read, otherwise None.
        """

        return B2bIo().readNefFile(fileName)

    @staticmethod
    def read_nef_file_sequence_shifts(fileName):
        """
        Note: this reader is limited, as it only handles the sequence and chemical shift information.
        It will need to be extended for other data types, if that should become necessary.

        :param fileName: Input NEF file path
        :return: dictionary with chainCode as key, then a list of sequence codes (pos 0) and a list of and shift information (pos 1)
        """

        return B2bIo().readNefFileSequenceShifts(fileName)



"""
======================COPYRIGHT/LICENSE START==========================

distanceConstraintsIO.py: I/O for nmrStar distance constraints saveframes

Copyright (C) 2005-2008 Wim Vranken (European Bioinformatics Institute)

=======================================================================

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

A copy of this license can be found in ../../../../license/LGPL.license

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA


======================COPYRIGHT/LICENSE END============================

for further information, please contact :

- CCPN website (http://www.ccpn.ac.uk/)
- PDBe website (http://www.ebi.ac.uk/pdbe/)

- contact Wim Vranken (wim@ebi.ac.uk)
=======================================================================

If you are using this software for academic purposes, we suggest
quoting the following references:

===========================REFERENCE START=============================
R. Fogh, J. Ionides, E. Ulrich, W. Boucher, W. Vranken, J.P. Linge, M.
Habeck, W. Rieping, T.N. Bhat, J. Westbrook, K. Henrick, G. Gilliland,
H. Berman, J. Thornton, M. Nilges, J. Markley and E. Laue (2002). The
CCPN project: An interim report on a data model for the NMR community
(Progress report). Nature Struct. Biol. 9, 416-418.

Wim F. Vranken, Wayne Boucher, Tim J. Stevens, Rasmus
H. Fogh, Anne Pajon, Miguel Llinas, Eldon L. Ulrich, John L. Markley, John
Ionides and Ernest D. Laue (2005). The CCPN Data Model for NMR Spectroscopy:
Development of a Software Pipeline. Proteins 59, 687 - 696.

===========================REFERENCE END===============================
"""

import os, string

from b2bTools.general.ccpn.universal.Io import getTopDirectory

# Import general functions
from b2bTools.general.ccpn.format.nmrStar.util import returnStarInt

from b2bTools.general.ccpn.format.nmrStar.generalIO import NmrStarConstraintFile
from b2bTools.general.ccpn.format.nmrStar.generalIO import NmrStarFile
from b2bTools.general.ccpn.format.nmrStar.generalIO import GenericConstraint

from b2bTools.general.ccpn.format.general.Constants import defaultMolCode
from b2bTools.general.ccpn.format.general.Util import getSeqAndInsertCode

#####################
# Class definitions #
#####################

class NmrStarFile(NmrStarFile):

  def initialize(self, version = '2.1.1'):

    self.constraintFiles = []
    self.files = self.constraintFiles

    if not self.version:
      self.version = version

    # Note: this will create problems for general_distance_constraints. TODO FIX!!
    self.saveFrameName = 'distance_constraints'
    self.DataClassFile = NmrStarDistanceConstraintFile

    self.components = [self.saveFrameName]
    self.setComponents()

  def read(self,verbose = 0):

    self.readComponent(verbose = verbose)

class NmrStarDistanceConstraintFile(NmrStarConstraintFile):

  def initialize(self,parent,saveFrame = None):

    self.constraints = []

    self.constraintElements = 2

    self.saveFrame = saveFrame

    self.parent = parent
    self.version = parent.version

    if self.saveFrame:
      self.parseSaveFrame()

  def parseSaveFrame(self):

    findConstraint = {}

    if not self.checkVersion():
      return

    #
    # Nodes & their logic relationships
    #

    tableTagNames = {}

    isGeneralType = False

    if self.version == '2.1.1':

      tableName = '_Constraint_ID'
      tableTagNames['constraintID'] = tableName
      tableTagNames['nodeID'] = '_Constraint_tree_node_ID'
      tableTagNames['logicOperation'] = '_Constraint_tree_logic_operation'
      tableTagNames['downNodeID'] = '_Constraint_tree_down_node_ID'
      tableTagNames['rightNodeID'] = '_Constraint_tree_right_node_ID'

    elif self.version == '3.0':

      tableName = '_Dist_constraint_tree'
      tableTagNames['constraintID'] = 'ID'
      tableTagNames['nodeID'] = 'Node_ID'
      tableTagNames['logicOperation'] = 'Logic_operation'
      tableTagNames['downNodeID'] = 'Down_node_ID'
      tableTagNames['rightNodeID'] = 'Right_node_ID'

    elif self.version == '3.1':

      tableName = '_Dist_constraint_tree'
      if self.saveFrame.tables.has_key(tableName):
        tableTagNames['constraintID'] = 'Constraint_ID'
        tableTagNames['nodeID'] = 'Node_ID'
        tableTagNames['logicOperation'] = 'Logic_operation'
        tableTagNames['downNodeID'] = 'Down_node_ID'
        tableTagNames['rightNodeID'] = 'Right_node_ID'
      else:
        isGeneralType = True
        tableName = "_Gen_dist_constraint"
        tableTagNames['constraintID'] = 'ID'

        tableTagNames['memberID'] = 'Member_ID'
        tableTagNames['memberLogic'] = 'Member_logic_code'

        tableTagNames['chainCode'] = 'Entity_assembly_ID_%s'
        tableTagNames['seqCode'] = 'Comp_index_ID_%s'
        tableTagNames['atomName'] = 'Atom_ID_%s'
        tableTagNames['resLabel'] = "Comp_ID_%s"
        tableTagNames['target'] = 'Distance_val'
        tableTagNames['lowerBound'] = 'Distance_lower_bound_val'
        tableTagNames['upperBound'] = 'Distance_upper_bound_val'
        tableTagNames['intensity'] = 'Intensity_val'

    #
    # Could be missing...
    #

    if self.saveFrame.tables.has_key(tableName):

      constraintTableTags = self.saveFrame.tables[tableName].tags

      for i in range(len(constraintTableTags[tableTagNames['constraintID']])):

        Id = constraintTableTags[tableTagNames['constraintID']][i]

        if not findConstraint.has_key(Id):
          self.constraints.append(NmrStarDistanceConstraint())
          if not isGeneralType:
            self.constraints[-1].setData(constraintTableTags,i,tableTagNames)
          findConstraint[Id] = self.constraints[-1]

        elif not isGeneralType:
          self.constraints[-1].addNode(constraintTableTags,i,tableTagNames)

        # Always have to add a node for general type
        if isGeneralType:
          self.constraints[-1].nodes.append(NmrStarDistanceConstraintItem(Id = constraintTableTags[tableTagNames['memberID']][i]))

        self.constraints[-1].setErrors(self.saveFrame.tables[tableName].tagErrors,tableTagNames,i)

        # Deal with things immediately, all info available.
        if isGeneralType:

          # TODO also handle logic here?

          node = self.constraints[-1].nodes[-1]

          if node.Id != '1':
            node.logicOperation = constraintTableTags[tableTagNames['memberLogic']][i]

          for index in ('1','2'):

            tmpTableTagNames = {'memberID': 'Member_ID'} # Note: this is reset!

            for tableTagName in tableTagNames.keys():
              if tableTagNames[tableTagName].count("%s"):
                tmpTableTagNames[tableTagName] = tableTagNames[tableTagName] % index

            # Reset member ID - the memberId in gen_dist is really the node ID (or item)
            node.addMember(constraintTableTags,i,tmpTableTagNames)
            node.members[-1].Id = index

          node.addDistance(constraintTableTags,i,tableTagNames)

      #
      # Node members
      #

      if not isGeneralType:

        tableTagNames = {}

        if self.version == '2.1.1':

          tableName = '_Constraint_tree_node_member_constraint_ID'
          tableTagNames['constraintID'] = tableName
          tableTagNames['nodeID'] = '_Constraint_tree_node_member_node_ID'

          tableTagNames['memberID'] = '_Constraint_tree_node_member_ID'
          tableTagNames['fracVal'] = '_Contribution_fractional_value'
          tableTagNames['chainCode'] = '_Mol_system_component_code'
          tableTagNames['seqCode'] = '_Residue_seq_code'
          tableTagNames['atomName'] = '_Atom_name'
          tableTagNames['resLabel'] = "_Atom_residue_label"

        elif self.version == '3.0':

          tableName = '_Dist_constraint'
          tableTagNames['constraintID'] = 'Dist_constraint_tree_ID'
          tableTagNames['nodeID'] = 'Tree_node_member_node_ID'

          tableTagNames['memberID'] = 'Constraint_tree_node_member_ID'
          tableTagNames['fracVal'] = 'Contribution_fractional_val'

          tableTagNames['chainCode'] = 'Label_entity_assembly_ID'
          tableTagNames['seqCode'] = 'Label_comp_index_ID'
          tableTagNames['atomName'] = 'Label_atom_ID'
          tableTagNames['resLabel'] = "Label_comp_ID"

          # 3.0 or higher only...
          tableTagNames['chemCompID'] = 'Label_comp_ID'

        elif self.version == '3.1':

          tableName = '_Dist_constraint'
          tableTagNames['constraintID'] = 'Tree_node_member_constraint_ID'
          tableTagNames['nodeID'] = 'Tree_node_member_node_ID'
          tableTagNames['memberID'] = 'Constraint_tree_node_member_ID'

          #tableTagNames['fracVal'] = 'Contribution_fractional_val'

          tableTagNames['chainCode'] = 'Entity_assembly_ID'
          tableTagNames['seqCode'] = 'Comp_index_ID'
          tableTagNames['atomName'] = 'Atom_ID'
          tableTagNames['resLabel'] = "Comp_ID"

          # 3.0 or higher only...
          tableTagNames['chemCompID'] = 'Comp_ID'


        if self.version[0] == '3' and not self.saveFrame.tables[tableName].tags[tableTagNames['atomName']][0]:

          if self.version == '3.0':
            tableTagNames['chainCode'] = 'Auth_segment_code'
          elif self.version == '3.1':
            tableTagNames['chainCode'] = 'Auth_asym_ID'

          tableTagNames['seqCode'] = 'Auth_seq_ID'
          tableTagNames['atomName'] = 'Auth_atom_ID'
          tableTagNames['resLabel'] = "Auth_comp_ID"

          # 3.0 or higher only...
          tableTagNames['chemCompID'] = 'Auth_comp_ID'

        nodeMembersTableTags = self.saveFrame.tables[tableName].tags

        for i in range(len(nodeMembersTableTags[tableTagNames['constraintID']])):

          Id = nodeMembersTableTags[tableTagNames['constraintID']][i]

          if not findConstraint.has_key(Id):
            print ("  Error in nmrStar file: constraint %s is not defined but is referenced." % Id)

          else:
            treeNodeId = nodeMembersTableTags[tableTagNames['nodeID']][i]
            constraint = findConstraint[Id]
            node = constraint.findNode(treeNodeId)
            node.addMember(nodeMembersTableTags,i,tableTagNames)

            constraint.setErrors(self.saveFrame.tables[tableName].tagErrors,tableTagNames,i)

        #
        # Distances (can be extended!)
        #

        tableTagNames = {}

        if self.version == '2.1.1':

          tableName = '_Distance_constraint_ID'
          tableTagNames['constraintID'] = tableName
          tableTagNames['nodeID'] = '_Distance_constraint_tree_node_ID'

          tableTagNames['target'] = '_Distance_value'
          tableTagNames['lowerBound'] = '_Distance_lower_bound_value'
          tableTagNames['upperBound'] = '_Distance_upper_bound_value'

        elif self.version == '3.0':

          tableName = '_Dist_constraint_value'
          tableTagNames['constraintID'] = 'Constraint_ID'
          tableTagNames['nodeID'] = 'Tree_node_ID'

          tableTagNames['target'] = 'Distance_val'
          tableTagNames['weight'] = 'Weight'
          tableTagNames['lowerBound'] = 'Distance_lower_bound_val'
          tableTagNames['upperBound'] = 'Distance_upper_bound_val'
          tableTagNames['intensity'] = 'Intensity_val'

        elif self.version == '3.1':

          tableName = '_Dist_constraint_value'
          tableTagNames['constraintID'] = 'Constraint_ID'
          tableTagNames['nodeID'] = 'Tree_node_ID'

          tableTagNames['target'] = 'Distance_val'
          tableTagNames['lowerBound'] = 'Distance_lower_bound_val'
          tableTagNames['upperBound'] = 'Distance_upper_bound_val'
          tableTagNames['intensity'] = 'Intensity_val'

        distanceTableTags = self.saveFrame.tables[tableName].tags

        for i in range(len(distanceTableTags[tableTagNames['constraintID']])):

          Id = distanceTableTags[tableTagNames['constraintID']][i]

          if not findConstraint.has_key(Id):
            print ("  Error in nmrStar file: constraint %s is not defined but is referenced." % Id)

          else:
            treeNodeId = distanceTableTags[tableTagNames['nodeID']][i]
            constraint = findConstraint[Id]
            node = constraint.findNode(treeNodeId)
            node.addDistance(distanceTableTags,i,tableTagNames)

            constraint.setErrors(self.saveFrame.tables[tableName].tagErrors,tableTagNames,i)

    #
    # Comments
    #

    self.parseCommentsLoop()

class NmrStarDistanceConstraint(GenericConstraint):

  def __init__(self, Id = None):

    self.Id = Id
    self.nodes = []
    self.errors = []

  def setData(self,constraintTableTags,i,tableTagNames):

    self.Id = constraintTableTags[tableTagNames['constraintID']][i]
    self.addNode(constraintTableTags,i,tableTagNames)

  def addNode(self,constraintTableTags,i,tableTagNames):

    self.nodes.append(NmrStarDistanceConstraintItem())
    self.nodes[-1].setData(constraintTableTags,i,tableTagNames)

  def addEmptyNode(self, Id = None):

    self.nodes.append(NmrStarDistanceConstraintItem(Id = Id))

  def findNode(self,treeNodeId):

    treeNodeId = returnStarInt(treeNodeId)
    for node in self.nodes:
      if treeNodeId == node.Id:
        return node

    print ("  TreenodeID %d not found!" % treeNodeId)

    return None

class NmrStarDistanceConstraintItem:

  #
  # Formerly TreeNode
  #

  def __init__(self, Id = None, constraintType = ""):

    self.Id = Id
    self.members = []
    self.type = constraintType
    self.logicOperation = None
    self.downId = None
    self.rightId = None

  def setData(self,constraintLogicTableTags,i,tableTagNames):

    self.Id = constraintLogicTableTags[tableTagNames['nodeID']][i]
    self.logicOperation = constraintLogicTableTags[tableTagNames['logicOperation']][i]
    self.downId = constraintLogicTableTags[tableTagNames['downNodeID']][i]
    self.rightId = constraintLogicTableTags[tableTagNames['rightNodeID']][i]

  def addMember(self,nodeMembersTableTags,i,tableTagNames):
    #
    # Relationship between multiple members with same ID in node always OR!!
    #
    self.members.append(NmrStarDistanceConstraintMember())
    self.members[-1].setData(nodeMembersTableTags,i,tableTagNames)

  def addEmptyMember(self,Id = None):

    self.members.append(NmrStarDistanceConstraintMember(Id = Id))

  def addDistance(self,distanceTableTags,i,tableTagNames):

    #
    # TODO: COULD link the class attribute name to the tableTagNames dict keys...
    # then run loop. Also have room for function for special cases!!!
    #

    self.type = 'Distance'
    self.target = distanceTableTags[tableTagNames['target']][i]
    self.lowerBound = distanceTableTags[tableTagNames['lowerBound']][i]
    self.upperBound = distanceTableTags[tableTagNames['upperBound']][i]

    self.intensity = distanceTableTags[tableTagNames['intensity']][i]

    if tableTagNames.has_key('weight'):
      self.weight = distanceTableTags[tableTagNames['weight']][i]

class NmrStarDistanceConstraintMember:

  #
  # Formerly TreeNodeMember
  #

  def __init__(self,Id = None, chainCode = None,seqCode = None, atomName = None, resLabel = None):

    self.Id = Id
    self.fracVal = None
    self.chainCode = chainCode
    (self.seqCode,self.seqInsertCode) = getSeqAndInsertCode(seqCode)
    self.atomName = atomName
    self.resLabel = resLabel

  def setData(self,nodeMembersTableTags,i,tableTagNames):

    self.Id = nodeMembersTableTags[tableTagNames['memberID']][i]

    if tableTagNames.has_key('fracVal'):
      self.fracVal = nodeMembersTableTags[tableTagNames['fracVal']][i]

    self.chainCode = nodeMembersTableTags[tableTagNames['chainCode']][i]
    if not self.chainCode:
      self.chainCode = defaultMolCode

    self.resLabel = nodeMembersTableTags[tableTagNames['resLabel']][i]
    (self.seqCode,self.seqInsertCode) = getSeqAndInsertCode(nodeMembersTableTags[tableTagNames['seqCode']][i])

    # This is a hack, is possible in NMR-STAR file from CNS/XPLOR data!
    if self.seqCode == None and self.chainCode:
      self.seqCode = 1

    self.atomName = nodeMembersTableTags[tableTagNames['atomName']][i]

    if tableTagNames.has_key('chemCompID'):
      self.chemCompID = nodeMembersTableTags[tableTagNames['chemCompID']][i]


###################
# Main of program #
###################

if __name__ == "__main__":

  files = [#['../reference/nmrStar/1e91.restraints','2.1.1'],
           #['../reference/ccpNmr/aartUtrecht/1byx/restraints.star','2.1.1'],
           #['../reference/ccpNmr/aartUtrecht/1bsh/restraints.star','2.1.1'],
           #['../reference/ccpNmr/jurgenBmrb/1d8b/restraints.star','2.1.1']
           #['../reference/ccpNmr/jurgenBmrb/1d8b/1d8b.str','3.0']
           ['../reference/nmrStar/tmp/tmp/1ak7/restraints_nonredun.str','3.0']
           #['../reference/ccpNmr/jurgenBmrb/1ao9/1ao9.str','3.0']
          ]
  for (file,version) in files:

    file = os.path.join(getTopDirectory(), file)

    nmrStarFile = NmrStarFile(file,version = version)

    nmrStarFile.read(verbose = 1)

    for constraintFile in nmrStarFile.constraintFiles:
      """
      for comment in constraintFile.comments:
        print comment
    """

      for constraint in constraintFile.constraints:
        if constraint.errors:
          print (constraint.errors)
        for node in constraint.nodes:
          """
          if hasattr(node,'weight') and node.weight:
            print node.weight,str(node.weight) # nmrStar 3.0 ONLY!!
          """
          if hasattr(node,'target'):
            print (constraint.Id, node.Id, node.target, node.lowerBound, node.upperBound)
          mlist = []
          for member in node.members:
            mlist.append([member.Id,member.seqCode,member.atomName])
          print ("   " + str(mlist))
          """

          """



#!/usr/bin/env python
# encoding: utf-8
"""
Predictor.py

Created by Wim Vranken on 1st May 2020, as wrapper around original Disomine code
Note that the original DisoMine code was upgraded to Python3.7 and simplified by taking out redundant code for production version
The DisoMine code was also modified to better work in conjunction with the data structures used here.
Copyright (c) 2020- Wim Vranken. All rights reserved.
"""

from b2bTools.general.Io import B2bIo
from b2bTools.singleSeq.DisoMine.standalone import standalone

class DisoMine(B2bIo):

  version = '2.0'
  # scriptName = "b2bTools.singleSeq.DisoMine.Predictor"
  scriptName = "python.b2bTools.singleSeq.DisoMine.Predictor"
  name = "DisoMine"
  def __init__(self):
    self.allPredictions = {}

    # Additional info for writing files
    self.references = ['doi: 10.1101/2020.05.25.115253 (2020)']
    self.infoTexts = ['Generated by DisoMine', 'See http://bio2byte.be']

    self.informationPerPredictor = {
      self.name: {
        'references': self.references,
        'info':       ";".join(self.infoTexts),
        'version':    self.version,
        'origin':     self.scriptName
      }
    }

  # Warning: THIS IS NOW NOT WORKING FROM A DICT BUT FROM A LIST OF TUPLES!
  def predictSeqs(self, seqs):

    """
    :param seqs: List/tuple of (seqId,sequenceString) tuples
    :return: True if all sequences predicted, False if problems

    See self.allPredictions for the predictions themselves.
    """

    assert hasattr(self,'allPredictions') and self.allPredictions and self.allPredictions != {}, "Have to run DynaMine and EFoldMine prior to DisoMine"

    disoMineResults = standalone(seqs, self.allPredictions)

    if not disoMineResults:
      print("DisoMine failed....")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
#  agmata.py
#
#  Copyright 2018 Gabriele Orlando <orlando.gabriele89@gmail.com>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#
#
import os, sys, gc
import pathlib, shutil, tempfile
import subprocess
import pickle

import numpy as np

from b2bTools.singleSeq.DisoMine.vector_builder.vettore_gen import build_vector


class AgmataRuntimeError(RuntimeError):
	pass


class agmata():

	def __init__(self,features='dyna_back,dyna_coil,dyna_sheet,dyna_helix,dyna_side',sw=1,verbose=2):
		self.features=features
		self.sw=sw
		self.LEARNING='log_reg'
		self.verbose=verbose
		self.params=[self.LEARNING,self.features,self.sw]
		self.scriptDir = str(pathlib.Path(__file__).parent.absolute())
		self.tmpDir = tempfile.mkdtemp()
		self.binDir = os.path.dirname(self.scriptDir)

	def load(self,force=False):

		with open(os.path.join(self.scriptDir,'..','marshalled','model_parameters.m'), 'rb') as f:
			u = pickle._Unpickler(f)
			u.encoding = 'latin1'
			p = u.load()
			if self.params!=p:
				if force==False:
					print('PARAMETERS CHANGED! NEW FITTING')
					self.fit()
				else:

					print('PARAMETERS CHANGED! but force==True,keeping old params(',p,')')
					# Dp,Da,Dn=self.strands=pickle.load(open('marshalled/discriminative.m','rb'))
					Dp,Da,Dn=self.strands=pickle.load(open('marshalled/agmata_discriminative_converted.m','rb'))
					self.Dp=Dp
					self.Da=Da
					self.Dn=Dn
					self.params=p
					self.LEARNING=p[0]
					self.features=p[1]
					self.sw=p[2]
			else:
				# with open(os.path.join(self.scriptDir,'..','marshalled','discriminative.m'),'rb') as f:
				with open(os.path.join(self.scriptDir,'..','marshalled','agmata_discriminative_converted.m'),'rb') as f:
					u = pickle._Unpickler(f)
					u.encoding = 'latin1'
					p = u.load()
					Dp,Da,Dn=p
					self.Dp=Dp
					self.Da=Da
					self.Dn=Dn

	def compare(self,v1,v2,COMP='vicini'):
		vf=[]
		#print v1,v2
		if COMP=='diff':

			if '-' in v1:
				vf=[]
				for i in v2:
					if type(i)!=str:
						vf+=[i]
				return vf
			elif '-' in v2:
				vf=[]
				for i in v1:
					if type(i)!=str:
						vf+=[i]
				return vf
			else:

				for i in range(len(v1)):

					if type(v1[i])!=str:
						vf+=[abs(v1[i]-v2[i])]

					else:
						if (v1[i].upper(),v2[i].upper()) in blosum:

							vf+=[blosum[(v1[i].upper(),v2[i].upper())]]
						else:

							vf+=[blosum[(v2[i].upper(),v1[i].upper())]]
		if COMP=='vicini':
			if '-' in v1:
				vf=[]
				for i in v2:
					if type(i)!=str:
						vf+=[i]
				return vf
			elif '-' in v2:
				vf=[]
				for i in v1:
					if type(i)!=str:
						vf+=[i]
				return vf
			else:

				for i in range(len(v1)):

					if type(v1[i])!=str:
						vf+=[v1[i],v2[i]]

					else:
						if (v1[i].upper(),v2[i].upper()) in blosum:

							vf+=[blosum[(v1[i].upper(),v2[i].upper())]]
						else:

							vf+=[blosum[(v2[i].upper(),v1[i].upper())]]
		return vf
	def predict(self,seqList,dmPredictions):
		features=self.features
		window= self.sw
		verbose=self.verbose

		currentPlatform = sys.platform
		if currentPlatform.startswith("linux"):
			agmata_suffix = "linux"
		elif currentPlatform == 'darwin':
			agmata_suffix = 'mac'
		elif currentPlatform == 'win32':
			agmata_suffix = 'windows'
		else:
			raise AgmataRuntimeError("Executable not available for this OS - please contact wim.vranken@vub.be to add it.")


		results = {}

		for i in range(len(seqList)):
			(proteinId,sequence) = seqList[i]
			if verbose>=2:
				print('\tStarting target:',proteinId)
			vet=np.array(build_vector(sequence,dmPredictions[proteinId],TYPE=features,sw=window))

			x=[]
			for i in range(len(vet)):
				for j in range(i,len(vet)):
					x+=[self.compare(vet[i],vet[j])]

			if self.verbose>=1:
				print('\tnumero coefficienti:',2*len(x))

			if self.LEARNING=='kde':
				par=self.Dp.score_samples(x)
				ant=self.Da.score_samples(x)
				non=self.Dn.score_samples(x)
				cont=0
				f=open(os.path.join(self.tmpDir,'coef.tmp'),'w')
				for i in range(len(vet)):
					for j in range(i+1,len(vet)):
						pa=-(par[cont]-non[cont])
						an=-(ant[cont]-non[cont])
						f.write(str(i+1)+' '+str(j+1)+' '+str(pa)+' '+str(an)+' 0.00000 0.00000 \n')
						cont+=1
				f.close()
			elif self.LEARNING=='log_reg' or self.LEARNING=='SVM':
				ant=self.Da.predict_log_proba(x)
				par=self.Dp.predict_log_proba(x)
				cont=0
				f=open(os.path.join(self.tmpDir,'coef.tmp'),'w')
				for i in range(len(vet)):
					for j in range(i,len(vet)):
						pa=-(par[cont][1]-par[cont][0])
						an=-(ant[cont][1]-ant[cont][0])
						f.write(str(i+1)+' '+str(j+1)+' '+str(pa)+' '+str(an)+' 0.00000 0.00000 \n')
						cont+=1
				f.close()

			else:
				raise('unknown discriminative method')

			with open(os.path.join(self.tmpDir,'seq.tmp'),'w') as f:
				f.write(sequence+'\n')

			# Run agmata binary as a subprocess in the temporary directory.
			# This directory should contain seq.tmp and coef.tmp.
			agmata_exe = "{0}/bin/agmata_c_final_{1}".format(self.binDir, agmata_suffix)
			agmataProc = subprocess.run(
							[agmata_exe, "seq.tmp", "600", "0"], cwd=self.tmpDir)
			if agmataProc.returncode != 0:
				raise AgmataRuntimeError("Execution of agmata failed (exit status {0:d})."
								   "".format(agmataProc.returncode))

			with open(os.path.join(self.tmpDir,'aggr_profile.dat'),'r') as f:
				output=[]
				for ln in f.readlines():
					output.append(float(ln.strip()))
			gc.collect()

			results[proteinId] = output
		shutil.rmtree(self.tmpDir)
		return results



__author__    = "$Author: wfv20 $"
___revision__ = "$Revision: 1.1 $"
___date__     = "$Date: 2003-07-01 12:56:12 $"

"""
Goal of these routines are to provide a Python interface to writting, reading,
analyzing, and modifying NMR-STAR files and objects.

NOTES:
* Not supported STAR features:
    - Nested loops
    - Global block
* Limitations to content:
    - STAR file should have one and only one data_ tag and that should
        be the first thing in the file
* Limitations to the lay out (for fast parsing) these can all be
    taken care of by first running the STAR file through formatNMRSTAR
    - Save frames should start and end with save_ at the beginning of
        the line
    - Perhaps some unknown;-(

SPEED ISSUES:
* There's a good Python API written by Jens Linge and Lutz Ehrlig (EMBL).
    It can handle much more STAR features and variations to content
    and lay out. The current API was written to handle NMR-STAR files in
    the order of several Mb for which the EMBL API demanded a lot of
    resources. Parsing a 1 Mb STAR file with a huge table of mostly numeric
    values required a peak 50 Mb in memory and about 2 hours with StarFormat.
    My guess was that this could be much faster if at least the lowest level
    of the dataNode value (where it is a string or number) would use native
    Python objects in stead.
    Another issue is that a large text object when parsed by the
    EMBL API got copied over and over resulting in loss of speed and a
    significant increase in memory use.
* This API uses native Python objects for a list of tags (looped or free)
    with user defined objects above that where speed and memory are less of an
    issue. It parses a 10 Mb STAR file in 25 seconds with a peak memory
    usage of 55 Mb. The average value in the file is 3 chars long. A Python
    string object has a reference count (4), type pointer (4), malloc overhead
    (4), trailing \0 (1) and the content (rounded up to multiples of 4).
    Ignoring the content rounding we go from 3 bytes to 20 bytes (factor 7)
    in total for the average string in the example file. Considering some
    overhead for the objects on top of the string objects the 55 Mb doesn't
    look that bad.
* Compare this with the C STARLIB2 from Steve Mading (BMRB) which takes 12
    cpu seconds and 18 Mb peak memory usage. For STARLIBJ (java) Steve
    got 40 Mb peak memory usage and 57 seconds. Memory usage is slightly
    better but speed is a factor 2 slower. This was using the best Java
    engine we had. Another one we tested was a factor 3 slower.
* Summary:

Test on Linux using a dual CPU 700 MHz Pentium III (urchin.bmrb.wisc.edu)
Language STAR file size (Mb) Time (s)  RAM (Mb) Notes
###############################################################################
C        10                  12        18       Using Steve's STARlib2.
Java     10                  57        40       Tested by Steve 
Python   10                  25        55       Written at BMRB
Python*  1*                  7200*     50*      Written at EMBL
###############################################################################
Labeled with asterisk because the size of test file had to be truncated.
The API was developed for small files (< 100 kb).

Test on Windows using a single CPU 500 MHz Pentium III (sepia2.bmrb.wisc.edu)
In this case not only cpu seconds were counted but also I/O etc.
Language STAR file size (Mb) Time (s)  RAM (Mb) Notes
###############################################################################
C        10                  -         -        No compiled version available
Java     10                  74        31       Tested by Jurgen using Sun's Java 1.3.1
Python   10                  45        59       Written at BMRB
###############################################################################


* References:
S. R. Hall and A. P. F. Cook. STAR dictionary definition language: initial specification.
    J.Chem.Inf.Comput.Sci. 35:819-825, 1995.
S. R. Hall and N. Spadacinni. The STAR file: detailed specifications.
    J.Chem.Inf.Comput.Sci. 34:505-508, 1994.
J. P. Linge, M. Nilges, and Ehrlich L. StarDOM: from STAR format to XML.
    J Biomol NMR, 1999.
N. Spadacinni and C. B. Hall. Star_base: accessing STAR file data.
    J.Chem.Inf.Comput.Sci. 34:509-516, 1994.
J. Westbrook and P. E. Bourne. STAR/mmCIF: An ontologoy for macromolecular structure.
    Bioinformatics. 16 (2):159-168, 2000.
"""

## Public attributes
verbosity               = 2



"""
======================COPYRIGHT/LICENSE START==========================

chemShiftsIO.py: I/O for nmrStar chemical shift saveframe

Copyright (C) 2005-2008 Wim Vranken (European Bioinformatics Institute)

=======================================================================

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

A copy of this license can be found in ../../../../license/LGPL.license

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA


======================COPYRIGHT/LICENSE END============================

for further information, please contact :

- CCPN website (http://www.ccpn.ac.uk/)
- PDBe website (http://www.ebi.ac.uk/pdbe/)

- contact Wim Vranken (wim@ebi.ac.uk)
=======================================================================

If you are using this software for academic purposes, we suggest
quoting the following references:

===========================REFERENCE START=============================
R. Fogh, J. Ionides, E. Ulrich, W. Boucher, W. Vranken, J.P. Linge, M.
Habeck, W. Rieping, T.N. Bhat, J. Westbrook, K. Henrick, G. Gilliland,
H. Berman, J. Thornton, M. Nilges, J. Markley and E. Laue (2002). The
CCPN project: An interim report on a data model for the NMR community
(Progress report). Nature Struct. Biol. 9, 416-418.

Wim F. Vranken, Wayne Boucher, Tim J. Stevens, Rasmus
H. Fogh, Anne Pajon, Miguel Llinas, Eldon L. Ulrich, John L. Markley, John
Ionides and Ernest D. Laue (2005). The CCPN Data Model for NMR Spectroscopy:
Development of a Software Pipeline. Proteins 59, 687 - 696.

===========================REFERENCE END===============================
"""

import os

from b2bTools.general.ccpn.format.nmrStar.generalIO import NmrStarGenericFile
from b2bTools.general.ccpn.format.nmrStar.generalIO import NmrStarFile

from b2bTools.general.ccpn.format.general.Util import getSeqAndInsertCode
from b2bTools.general.ccpn.format.general.Constants import defaultMolCode

#####################
# Class definitions #
#####################

class NmrStarFile(NmrStarFile):

  def initialize(self, version = '2.1.1'):

    self.chemShiftFiles = []

    self.files = self.chemShiftFiles

    if not self.version:
      self.version = version

    self.saveFrameName = 'assigned_chemical_shifts'
    self.DataClassFile = NmrStarChemShiftFile

    self.components = [self.saveFrameName]
    self.setComponents()

  def read(self,verbose = 0):

    if os.path.exists(self.name):
      text = open(self.name,'r').read()
      if not text.count(self.saveFrameName) and text.count("loop_"):

         # Also set version - can't do this from info
        if text.count('_Atom_chem_shift'):
          sfCatText = "   _Assigned_chem_shift_list.Sf_category       assigned_chemical_shifts"
        else:
          sfCatText = "   _Saveframe_category               assigned_chemical_shifts"

        # Try to fix the missing saveframe, and set this as the text.
        loopIndex = text.index("loop_")

        # Also try to catch saveframe indicator
        if text[:loopIndex].count("save_"):
          saveFrameStart = ""
        else:
          saveFrameStart = "save_shifts\n"

        saveFrameStart += "%s\n  loop_" % sfCatText
        text = text.replace("loop_",saveFrameStart,1)

        if not self.patt[self.format + 'EndSaveTag'].search(text):
          text += "save_\n"

        self.text = text
        print (self.text)
        print ("Warning: fixing input file to be correct NMR-STAR. This might not work.")

    self.readComponent(verbose = verbose)

class NmrStarChemShiftFile(NmrStarGenericFile):

  """
  Information on file level
  """

  def initialize(self,parent,saveFrame = None):

    # 2.1 doesn't use this attribute.

    self.attrToTagMappings = (

      ('details','Details',None),
      ('chemShiftRefId','Chem_shift_reference_ID',None),

    )

    self.chemShifts = []

    self.saveFrame = saveFrame

    self.parent = parent
    self.version = parent.version

    if self.saveFrame:
      self.parseSaveFrame()

  def parseSaveFrame(self):

    if not self.checkVersion():
      return

    if self.version == '2.1.1':
      # In case is missing in loop_ only shift files.
      if '_Mol_system_component_name' in self.saveFrame.tags:
        molCode = self.saveFrame.tags['_Mol_system_component_name']
      else:
        molCode = defaultMolCode

      if '_Atom_shift_assign_ID' not in self.saveFrame.tables.keys():
        # Possible apparently
        return

      chemShiftTableTags = self.saveFrame.tables['_Atom_shift_assign_ID'].tags
      numChemShifts = len(chemShiftTableTags['_Atom_shift_assign_ID'])
    else:
      if self.attrToTagMappings:

        for (attrName,tagName,default) in self.attrToTagMappings:
          if tagName in self.saveFrame.tags:
            attrValue = self.saveFrame.tags[tagName]
          else:
            attrValue = default

          if not hasattr(self,attrName) or not self.attrName:
            setattr(self,attrName,attrValue)

      molCode = None
      chemShiftTableTags = self.saveFrame.tables['_Atom_chem_shift'].tags
      if 'ID' in chemShiftTableTags:
        numChemShifts = len(chemShiftTableTags['ID'])
      elif 'Comp_ID' in chemShiftTableTags:
        numChemShifts = len(chemShiftTableTags['Comp_ID'])
      else:
        numChemShifts = 0


    for i in range(0,numChemShifts):

      tmpMolCode = molCode
      if not tmpMolCode:
        # TODO could in principle find entity_assembly name (or code)
        # What is the best way to handle this?
        if 'Entity_assembly_ID' in chemShiftTableTags:
          tmpMolCode = str(chemShiftTableTags['Entity_assembly_ID'][i])
        else:
          tmpMolCode = defaultMolCode

      self.chemShifts.append(NmrStarChemShift(tmpMolCode,self))
      self.chemShifts[-1].setData(chemShiftTableTags,i)

    tableName = '_Chem_shift_experiment'
    self.setMeasureExperiments(tableName)

    tableName = '_Chem_shift_software'
    self.setMeasureSoftwares(tableName)

class NmrStarChemShift:

  def __init__(self,molCode,parent):

    self.molCode = molCode[:65]  # Make sure this is truncated! Used as resonance name later on...
    self.parent = parent
    self.seqCode = None

  def setData(self,chemShiftTableTags,i):

    # Values already formatted...

    if self.parent.version == '2.1.1':
      assignList = [['Id',       '_Atom_shift_assign_ID',None],
                    ['seqCode',  '_Residue_seq_code',None],
                    ['resLabel', '_Residue_label',None],
                    ['atomName', '_Atom_name',None],
                    ['atomType', '_Atom_type',None],
                    ['value',    '_Chem_shift_value',None],
                    ['valueError', '_Chem_shift_value_error',0.0],
                    ['ambCode',    '_Chem_shift_ambiguity_code',1],
                    ['ambCode',    '_Chem_shift_ambiguity_type',1], # This apparently also occurs sometimes...
                    ['authorSeqCode', '_Residue_author_seq_code',None]
                   ]
    else:
      # TODO can get a lot more out...
      assignList = [['Id',            'ID',None],
                    ['seqCode',       'Seq_ID',None],
                    ['backupSeqCode', 'Comp_index_ID',None],
                    ['resLabel',      'Comp_ID',None],
                    ['atomName',      'Atom_ID',None],
                    ['atomType',      'Atom_type',None],
                    ['value',         'Val',None],
                    ['valueError',    'Val_err',0.0],
                    ['figOfMerit',    'Assign_fig_of_merit',None],
                    ['ambCode',       'Ambiguity_code',1],
                    ['details',       'Details',None],
                    ['authorSeqCode', 'Author_seq_ID',None]
                   ]


    for (attrName,tagName,default) in assignList:

      if tagName in chemShiftTableTags:
        if chemShiftTableTags[tagName][i] != None:
          setattr(self,attrName,chemShiftTableTags[tagName][i])
        else:
          setattr(self,attrName,default)

    # Little hack - not sure what is best in the end...
    if not self.seqCode and hasattr(self,'backupSeqCode') and self.backupSeqCode:
      self.seqCode = self.backupSeqCode

    # For completeness...
    (self.seqCode,self.seqInsertCode) = getSeqAndInsertCode(self.seqCode)

    # This is a hack, is possible in older NMR-STAR files! Use resLabel as chain ID
    if self.seqCode == None and hasattr(self,'resLabel') and self.resLabel:
      self.seqCode = 1
      self.chainCode = self.resLabel



"""
Classes for dealing with STAR syntax
"""

__author__    = "$Author: jurgenfd $"
___revision__ = "$Revision: 1.8.4.2 $"
___date__     = "$Date: 2011-04-06 09:00:38 $"

"""
$Log: not supported by cvs2svn $
Revision 1.8.4.1  2010/02/11 12:44:33  wimvranken
Fix to deal with loop_ only chemical shift files - these are now modified on-the-fly in order to read them.

Revision 1.8  2007/07/09 12:22:27  wimvranken
Merge with branch 4.

Revision 1.7.4.1.4.2  2007/06/13 08:35:52  wimvranken
Removed copyright - is not CCPN

Revision 1.7.4.1.4.1  2006/10/25 10:57:09  wimvranken
Fixed problem that shows up in windows only (\ to continue end of line)

Revision 1.7.4.1  2006/07/21 17:29:01  wimvranken
Now only contain full path imports

Revision 1.7  2005/11/03 14:06:39  wimvranken
Replaced var text_lenght by text_length, added extra error message printout

Revision 1.6  2005/06/27 16:43:41  wb104
Updated licenses.

Revision 1.5  2004/12/15 17:57:20  tjs23
TJS: Updated licenses.

Revision 1.4  2003/08/08 12:45:54  wfv20
Changed preferred quote, added comment writing support

Revision 1.3  2003/07/14 09:34:32  wfv20
Modified so list references are not carried through class initialization

Revision 1.2  2003/07/10 16:13:31  wfv20
Changed to new universal setup

Revision 1.1  2003/07/01 12:56:18  wfv20
Jurgen Doreleijers modified Python nmrStar reader. Bug fixes and additions (notably for reading nmrView star files) have been made.

Revision 1.1.1.1  2001/11/02 20:16:40  jurgen
Initial package capable of read/write access to STAR files without nested loops

"""
## Standard modules
import os

## BMRB modules

if os.name != 'posix':
    tempdir     = None # The module tempfile will pick one
else:
    tempdir     = '/tmp'

# No changes required after this line
###############################################################################

"""
STAR file
Only methods for reading and writing are currently implemented.
datanodes is a list of possibly mixed saveframes and tagtables
"""

from .Utils import Lister
from .Text import *
from .SaveFrame import SaveFrame
from .TagTable import TagTable


class File (Lister):
    def __init__( self,
                    title                   = 'general_star_file_title',
                    filename                = '',
                    datanodes               = None,
                    flavor                  = None, # Call set_flavor when changing
                    preferred_quote         = '"',  # Put somewhere else?
                    verbosity   = 2
                  ):
        self.title      = title
        self.filename   = filename

        if datanodes:
          self.datanodes  = datanodes
        else:
          self.datanodes = []

        self.flavor     = flavor
        self.verbosity  = verbosity

    "Simple checks on integrity"
    def check_integrity( self,  recursive = 1  ):
        if recursive:
            for datanode in self.datanodes:
                if datanode.check_integrity( recursive = 1):
                    print("ERROR: integrity check failed for Saveframe")
                    return 1
        if self.verbosity >= 9:
            print('Checked integrity of File    (%2s datanodes,  recurs.=%s)  : OK [%s]' % (
                len(self.datanodes), recursive, self.title ))

    "Returns the STAR text representation"
    def star_text(self, flavor = None):
        if flavor == None:
            flavor = self.flavor
        str = 'data_%s\n' % self.title
        # Data node objects can be of type SaveFrame OR TagTable only
        # Data node object can now also contain comment information
        #      these comments are printed before the saveframe (Wim 2003/08/05)
        for datanode in self.datanodes:
            str = str + datanode.comment
            str = str + datanode.star_text( flavor = flavor)
        return str


    """
    Reads a NMR-STAR formatted file using
    the filename attribute.

    Added option to pass text in straight away - this is for handling crappy loop_ only files so
    I can fix them in memory (Wim 11/02/10).
    """

    def read (self, strip_comments=1, nmrView_type = 0, text = ""):

        if not text:
          if not self.filename:
              print('ERROR: no filename in STARFile with title:', self.title)
              return 1
          text = open(self.filename,'r').read()

        if self.parse( text=text, strip_comments=strip_comments, nmrView_type = nmrView_type):
            print("ERROR: couldn't parse file")
            return 1

        return 0

    """
    - Parses text into save frames and tagtables.
    - Input text should start at position given with non-white space character
    - Appends a list of datanodes(save frames or tagtables)
    """
    def parse (self, text='', strip_comments=1, nmrView_type = 0):

        if self.verbosity > 1:
            print('Parsing STAR file:', self.filename)

        """
        '"Begin at the beginning," the King said, gravely,
        "and go on till you come to the end; then stop."' (LC)
        """

        ## Collapse the semicolon block for ease of parsing
        ## Very expensive to do
        ## Timed at: xx seconds for a xx Mb file with xx semicolon blocks
        text = semicolon_block_collapse( text )

        ## Now it's easy to strip comments
        if strip_comments:
            text = comments_strip( text )


        ## For nmrView 'nmrStar' also compress {  } into {}
        ## Wim 05/03/2003

        if nmrView_type:
            text = nmrView_compress( text )

        ## TITLE
        match_data_tag = re.search(r'\s*data_(\S+)\s+', text, 0 )
        if not match_data_tag:
            print("Warning: Found no 'data_title' string in file's text.")
            print("Warning: Your file is not valid NMR-STAR - to attempt reading")
            print("Warning: this file a data_title tag was added automatically.")
            text = "data_autoTitle\n" + text
            match_data_tag = re.search(r'\s*data_(\S+)\s+', text, 0 )

        self.title = match_data_tag.group(1)
        pos = match_data_tag.end()

        ## Four quick searches for possible continuations
        next_sf_begin   = None      # SAVE FRAME BEGIN
        next_sf_end     = None      # SAVE FRAME END
        next_free_tt    = None      # FREE TAGTABLE
        next_loop_tt    = None      # LOOP TAGTABLE
        sf_open         = None      # When a saveframe is open
        text_length     = len(text)

        ## Only break when parsed to the eof
        while pos < text_length:
            if self.verbosity >= 9:
                print('Parse text from position:%s : [%s]' % (
                    pos, text[pos:pos+10] ))

            match_save_begin_nws = pattern_save_begin_nws.search(text,pos,pos+len('save_1'))
            if match_save_begin_nws:
                if match_save_begin_nws.start() == pos:
                    next_sf_begin = 1
            if not (next_sf_begin):
                match_save_end_nws = pattern_save_end_nws.search(text,pos,pos+len('save_ '))
                if match_save_end_nws:
                    if match_save_end_nws.start() == pos:
                        next_sf_end = 1
            if not (next_sf_begin or next_sf_end):
                match_tag_name_nws = pattern_tag_name_nws.search(text,pos,pos+len(' _X'))
                if match_tag_name_nws:
                    if match_tag_name_nws.start() == pos:
                        next_free_tt = 1
            if not (next_sf_begin or next_sf_end or next_free_tt):
                match_tagtable_loop_nws = pattern_tagtable_loop_nws.search(text,pos,pos+len('loop_ '))
                if match_tagtable_loop_nws:
                    if match_tagtable_loop_nws.start() == pos:
                        next_loop_tt = 1

            ## Just checking
            if not ( next_sf_begin or next_sf_end or next_free_tt or next_loop_tt ):
                print('ERROR: No new item found in data_nodes_parse.')
                print('Items looked for are a begin or end of a saveframe, or')
                print('a begin of a tagtable(free or looped).')
                print()
                print("At text:")
                print(text[pos:pos+70])
                print("Preceded by:")
                print(text[pos-200:pos])
                return None

            ## SAVE FRAME BEGIN
            if next_sf_begin:
                if sf_open:
                    print("ERROR: Found the beginning of a saveframe but")
                    print("ERROR: saveframe before is still open(not closed;-)")
                    return None
                match_save_begin = pattern_save_begin.search( text, pos )
                if not match_save_begin:
                    print("ERROR: Code error (no second match on sf begin)");
                    return None
                if match_save_begin.start() != pos:
                    print("ERROR: Code error (wrong second match on sf begin)");
                    return None
                self.datanodes.append( SaveFrame(  tagtables    = [], # Need resetting
                                            verbosity    = self.verbosity ) )
                self.datanodes[-1].title = match_save_begin.group(1)
                sf_open         = 1
                next_sf_begin   = None
                pos             = match_save_begin.end()
                continue

            ## SAVE FRAME END
            if next_sf_end:
                if not sf_open:
                    print("ERROR: Found the end of a saveframe but")
                    print("ERROR: saveframe was not open")
                    return None
                match_save_end = pattern_save_end.search( text, pos )
                if not match_save_end:
                    print("ERROR: Code error (no second match on sf end)");
                    return None
                if match_save_end.start() != pos:
                    print("ERROR: Code error (wrong second match on sf end)");
                    return None
                sf_open     = None
                next_sf_end = None
                pos         = match_save_end.end()
                continue

            ## FREE or LOOP TAGTABLE
            if next_free_tt:
                free            = 1
                next_free_tt    = None
            else: # next_loop_tt must be true as this was checked before
                if not next_loop_tt:
                    print('ERROR: code bug in File.parse()')
                    return None
                free            = None
                next_loop_tt    = None

                match_tagtable_loop = pattern_tagtable_loop.search( text, pos )
                if not match_tagtable_loop:
                    print('ERROR: Code error, no second match on tagtable_loop')
                    return None
                if match_tagtable_loop.start() != pos:
                    print("ERROR: Code error (wrong second match on tagtable_loop)")
                    return None
                pos = match_tagtable_loop.end()

            if sf_open:
                dn = self.datanodes[-1].tagtables # Insert in last saveframes' tagtables
            else:
                dn = self.datanodes

            dn.append(
                    TagTable(   free      = free,
                                tagnames  = [],
                                tagvalues = [],
                                verbosity = verbosity ) )
            tt = dn[-1] # Just to be verbose for the beloved reader
            pos = tt.parse( text=text, pos=pos )

            if pos ==  None:
                print("ERROR: In parsing tagtable")
                return None
            if self.verbosity >=9:
                print('Parsed tagtable up to pos: [%s]' % pos)

        if self.verbosity >= 9:
            print('Parsed: [%s] datanodes (top level count only)' % len( self.datanodes ))

        if self.check_integrity( recursive = 0):
            print("ERROR: integrity not ok")
            return 1

        # Save some memory
        text = ''
        return 0



    """
    Writes the object to a STAR formatted file using
    the filename attribute.
    """
    def write (self):
        if not self.filename:
            print('ERROR: no filename in STARFile with title:', self.title)
            return 1
        open(self.filename,'w').write( self.star_text() )
        if self.verbosity > 1:
            print('Written STAR file:', self.filename)


    """
    Tries to reformat a file on disk with the filename given in the
    attribute of this object.
    Running Steve Madings (BMRB) formatNMRSTAR program if available
    NOTE: this does NOT do anything with the datanodes of this object!
    """
    def formatNMRSTAR( self,
                    comment_file_str_dir    = '/bmrb/lib',
                    ):

        if self.verbosity >= 9:
            print("Attempting to reformat STAR file using external program if available")

        if os.name != 'posix':
            print("WARNING: No external program available on non-posix systems for reformatting STAR files")
            return 1

        ##  Try command and check for non-zero exit status
        ##  Note that these commands are only valid on Unix
        ##  Standard error is thrown on the bit bucket.
        cmd = "%s < %s 2>/dev/null" % ('formatNMRSTAR', self.filename)
        pipe = os.popen( cmd )
        output = pipe.read()

        ##  The program exit status is available by the following construct
        ##  The status will be the exit number (in one of the bytes)
        ##  unless the program executed successfully in which case it will
        ##  be None.
        status = pipe.close()
        if self.verbosity >= 9:
            print("Got status:", status)

        ## Success
        if ( status == None ):
            try:
                open(self.filename, 'w').write(output)
            except IOError:
                print('ERROR: Could not open the file for writing', self.filename)
                return 1
            if self.verbosity >= 9:
                print("Reformatted STAR file:", self.filename)
            return 0
        else:
            if self.verbosity :
                print("WARNING: Not pretty printing STAR file", self.filename)
            return 1


################################################################################
if __name__ == '__main__':
    if 1:
        # 0 is only errors, 1 is warnings as well, 2 is normal and 9 is debug
        STAR.verbosity              = 2
        strf                        = File( verbosity=STAR.verbosity)

        if os.name == 'posix':
            pathname = ''
        else:
            pathname = r'C:\Temp'

##        filename = 'local/test_nopound.txt'
##        filename = 'local/block_21921.txt'

        strf.filename = filename

##        def myfunc():
        if strf.read():
            print("ERROR: In read. Exiting program")


        # strf.filename
        # strf.formatNMRSTAR: REFORMATS FILE! Don't use!!
        # strf.attrnames:     bulk output of everything in file
        # strf.flavor:        mmCIF, ... (in principle only nmrStar works?)
        # strf.title:         pdb code?
        #
        # strf.datanodes:     list of saveframes
        #
        #               .title:       saveframe name
        #               .tagtables:   list o/ebi/msd/nmrqual/rawdata/8/1/1/info.general/bmr5106.stf tags in saveframe
        #                         .tagnames:  list of tagnames
        #                         .tagvalues: list of list of values for tagnames
        #                         .free:      if None, is a LOOP tagtable! If 1, is a list of simple tags
        #

        # time.time()
        # strf.read()
        # time.time()

##        profile.run( 'myfunc()' )

##        print "Exiting program (test done)"
##        sys.exit(0)


##            strf.flavor = 'mmCIF'
        #strf.filename = strf.filename + '_new1.str'
        #if strf.write():
        #    print "ERROR: In write. Exiting program"



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  shiftcrypt.py
#
#  Copyright 2018  <@gmail.com>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#
#
import warnings
import torch
import argparse, sysconfig
import pathlib
import sys
import os

PATH = str(pathlib.Path(__file__).parent.absolute())
sys.path.append(os.path.abspath(os.path.join(PATH, "../../..")))

from b2bTools.nmr.shiftCrypt.shiftcrypt_pkg import parser

class Standalone():

	PATH = PATH

	def run_shiftcrypt(self):

		warnings.filterwarnings("ignore")

		pa = argparse.ArgumentParser()

		pa.add_argument('-i', '--infile',
							help='the input Nmr Exchange Format (NEF) file',
							)
		pa.add_argument('-fit','--fit',
							help='Custom_model_Name. re-runs the training and saves the model as marshalled/Custom_model_Name. For custom model',

							default=None
							)
		pa.add_argument('-o', '--outfile',
							help='output file. Residues with a shiftcrypt value of -10 highlight residues with too many missing values',
							default=None)

		pa.add_argument('-m', '--model',
							help='model to use:\n\t1 --> full model\n\n\t2 --> model that uses just the most common atoms (default) \n\n\t3 --> the method with only N, H  and CA CS. Used for dimers. if you want to load a custom model,use -m NameOfTheModel. Please remember the model is required to be in ./marshalled/ and can be generated with python shiftcrypt.py --fit MOdelName ',
							default='2')
		args = pa.parse_args()

		if args.fit:

			#
			# This part generates new models based on a new dataset
			#

			from .src.chemical_shifts_custom_model import shifts
			from .src.autoencoder_standalone_version import autoencoder_transformation

			a=autoencoder_transformation(shifts)
			a.fit(training_folder='src/datasets/rereferenced_nmr/')
			torch.save(a,'marshalled/'+args.fit)
			print ('############################################')
			print ('TRAIN SUCCESSFUL! the model has been saved')
			print ('in the "marshalled" folder. To use it run')
			print ('python2 shiftcrypt.py -m ModelName -i input')
			print ('############################################')
			print ('The monkeys are listening')
			return

		if args.infile:
			try:
				allProteinShifts = parser.parse_official(fil=args.infile)
			except:
				print('error in the parsing of the file. Please double check the format. If everyhting is correct, please report the bug to @gmail.com')
				return

			self.loadModel(args.model, args)
			encryptedValues = self.encryptProteinShifts(allProteinShifts)

			(void,inputFile) = os.path.split(args.infile)

			output = self.organiseData(encryptedValues,allProteinShifts,inputFile=inputFile)

			# Dump to custom file or print
			if args.outfile!=None:
				f=open(args.outfile,'w')

				for entry in output:
					chainCode = entry['chainCode']
					sequence = entry['sequence']
					shiftCryptValues = entry['shiftCrypt']
					seqCodes = entry['seqCodes']

					for i in range(len(sequence)):
						f.write("{} {:5d} {:s} {:7.3f}\n".format(chainCode,seqCodes[i],sequence[i],shiftCryptValues[i]))
				f.close()

			else:
				print (output)
		else:
			print('-i or --infile argument is obligatory, please retry')

	def api_shiftcrypt(self, fileName, modelClass, is_star, original_numbering):

		"""
    :param proteinShifts: Shift information for a protein
    """

		proteinShifts = parser.parse_official(fileName,is_star=is_star,original_numbering=original_numbering)

		self.loadModel(modelClass, None)

		encryptedValues = self.encryptProteinShifts(proteinShifts)
		shiftCryptResults = self.organiseData(encryptedValues, proteinShifts,inputFile=os.path.split(fileName)[1])

		if not shiftCryptResults:
			print("ShiftCrypt failed....")

		return {'results': shiftCryptResults}

	def encryptProteinShifts(self,allProteinShifts):

		outputs = []

		try:
			print('Running transformation')
			print(allProteinShifts)
			for proteinShifts in allProteinShifts:
				outputs.append(self.auto.transform(proteinShifts))
		except:
			print('Error transforming the CS data. Please double-check you installed all the dependencies. If everyhting is correct, please report the bug to wim.vranken@vub.be')

		return outputs

	def loadModel(self, modelType, args):

		# Hacks to ensure the model is loaded, was saved within specific directory architecture
		sys.path.append(os.path.abspath(self.PATH))

		if modelType == '1':
			self.auto = torch.load(self.PATH + '/models/new_full.mtorch',
														 encoding='latin1')  # the method with the full set of Cs. this may retur a lot of -10 (missing values) because of the scarcity of cs data for some residues
		elif modelType == '2':
			self.auto = torch.load(self.PATH + '/models/new_commons.mtorch',
														 encoding='latin1')  # the method with just the most common Cs values
		elif modelType == '3':
			self.auto = torch.load(self.PATH + '/models/new_NH.mtorch',
														 encoding='latin1')  # the method with only N and H CS. Used for dimers
		else:
			try:
				self.auto = torch.load(self.PATH + '/models/' + args.new_model, encoding='latin1')
			except:
				print("custom model not found")
				self.auto = None

	def organiseData(self, encryptedValues, allProteinShifts,inputFile=None):

		# Now organise output into a decent dictionary structure
		output = []

		for protIndex in range(len(encryptedValues)):
			encryptedValue = encryptedValues[protIndex]
			protInfo = allProteinShifts[protIndex]

			sequence = []
			shiftcrypt = []
			seqCodes = []

			for i in range(len(protInfo.seq)):
				sequence.append(protInfo.seq[i])
				shiftcrypt.append(str(encryptedValue[i]))
				seqCodes.append(protInfo.resnum[i])

			entry = {}

			entry['ID_file'] = inputFile
			entry['sequence'] = sequence
			entry['seqCodes'] = seqCodes
			entry['shiftCrypt'] = shiftcrypt
			entry['chainCode'] = protInfo.chainCode

			output.append(entry)

		return output

if __name__ == '__main__':

	sa = Standalone()
	sa.run_shiftcrypt()



import json
import numpy as np
import matplotlib.pyplot as plt
# import matplotlib
# matplotlib.use('Agg')
# import matplotlib.pyplot as plt

# Call the function below to plot the results
# self.plot_results(json_file_names, mutation)



class Plotter:
	def __init__(self):
		self.biophys_dict = {1: 'backbone', 2: 'sidechain', 3: 'coil',
							 4: 'sheet', 5: 'helix', 6: 'earlyFolding',
							 7: 'disoMine'}

	def plot_msa_distrib(self, jsondata_list, mutation=False):
		colors = ['blue', 'orange']
		opt = int(input('Show plots or save the images?\n'
						'(0 = show; 1 = save)>>>'))
		# These for loops got too complicated, I have to think
		# something simpler to handle the None values in the data
		for biophys_data in jsondata_list[0]["results"].keys():
			for data, col in zip(jsondata_list, colors):
				none_idx = []
				for n in range(len(data['results'][biophys_data]['median'])):
					if data['results'][biophys_data]['median'][n] == None \
							or data['results'][biophys_data][
						'firstQuartile'][n] == None \
							or data['results'][biophys_data][
						'thirdQuartile'][n] == None:
						none_idx.append(n)

				range_list = []
				for n in range(len(none_idx)):
					try:
						if none_idx[n] + 1 != none_idx[n + 1]:
							range_list.append(
								(none_idx[n] + 1, none_idx[n + 1]))
						else:
							continue
					except:
						if len(none_idx) == 1:
							range_list.append((0, none_idx[0]))
							range_list.append((none_idx[0] + 1, len(
								data['results'][biophys_data][
									'median'])))

						else:
							range_list.append((0, none_idx[0]))
							range_list.append((none_idx[-1] + 1, len(
								data['results'][biophys_data][
									'median'])))

				# When there are None values in the data
				if range_list:
					for tuple in range_list:
						x = np.arange(tuple[0], tuple[1], 1)
						firstq = \
							data['results'][biophys_data][
								'firstQuartile'][
							tuple[0]:tuple[1]]
						thirdq = \
							data['results'][biophys_data][
								'thirdQuartile'][
							tuple[0]:tuple[1]]
						bottom = \
							data['results'][biophys_data][
								'bottomOutlier'][
							tuple[0]:tuple[1]]
						top = \
							data['results'][biophys_data]['topOutlier'][
							tuple[0]:tuple[1]]
						plt.fill_between(
							x, firstq, thirdq, alpha=0.5, color=col)
						plt.fill_between(
							x, bottom, top, alpha=0.25, color=col)

				# When there aren't None values in the data
				else:
					x = np.arange(0, len(
						data['results'][biophys_data]['median']), 1)
					firstq = data['results'][biophys_data][
						'firstQuartile']
					thirdq = data['results'][biophys_data][
						'thirdQuartile']
					bottom = data['results'][biophys_data][
						'bottomOutlier']
					top = data['results'][biophys_data]['topOutlier']
					plt.fill_between(
						x, firstq, thirdq, alpha=0.5, color=col)
					plt.fill_between(
						x, bottom, top, alpha=0.25, color=col)

				plt.plot(data['results'][biophys_data]['median'],
						 linewidth=1, color=col)

				if mutation:
					print(mutation)
					plt.plot(mutation['results'][biophys_data],
							 linewidth=0.5, color='red')

			plt.axis(
				[0, len(data['results'][biophys_data]['median']), 0, 1.1])
			plt.ylabel(biophys_data)
			plt.xlabel('Residue position')

			# if opt == 0:
			plt.show()
			# if opt == 1:
			# 	filename = file.split('.')[0] + '_' + biophys_data + '.png'
			# 	plt.savefig(filename)
			# 	print(
			# 		'plot succesfully saved in results/{}'.format(filename))
			# 	plt.close()

	def plot_json(self, jsondata):
		opt = int(input('Show plots or save the images?\n'
						'(0 = show; 1 = save)>>>'))

		for biophys_data in jsondata[list(jsondata.keys())[0]].keys():
			for result_id in jsondata.keys():
				if result_id != "sequence":
					plt.plot(jsondata[result_id][biophys_data],
							 label=result_id)
					plt.legend()
					plt.title(result_id)
					plt.ylabel(biophys_data)
					plt.xlabel('Residue position')

			# if opt == 0:
			plt.show()

			# if opt == 1:
			# 	filename = json_file_names.split(".")[
			# 				   0] + '_' + biophys_data + '.png'
			# 	plt.savefig("results/" + filename)
			# 	print('plot succesfully saved in results/{}'.format(filename))
			# 	plt.close()

		def plot_sinlge_seq(json_file_names, msa_like, mutation):
			with open(json_file_names) as json_file:
				data = json.load(json_file)
				opt = int(input('Show plots or save the images?\n'
							  '(0 = show; 1 = save)>>>'))

				for result_id in data['results']:
					for biophys_data in self.biophys_dict.values():
						plt.plot(result_id[biophys_data], label = result_id['proteinID'])
						plt.legend()
						plt.title(data['id'])

						plt.ylabel(biophys_data)
						plt.xlabel('Residue position')

						if opt == 0:
							plt.show()

						if opt == 1:
							filename = result_id['proteinID'] + '_' + biophys_data + '.png'
							plt.savefig("results/" + filename)
							print('plot succesfully saved in results/{}'.format(filename))
							plt.close()



from b2bTools.singleSeq.DynaMine.Predictor import DynaMine
from b2bTools.singleSeq.EFoldMine.Predictor import EFoldMine
from b2bTools.singleSeq.DisoMine.Predictor import DisoMine
from b2bTools.singleSeq.Agmata.Predictor import Agmata

from b2bTools.general.Io import B2bIo

# TODO: Here access all predictions, or at least those asked for!

class MineSuite(B2bIo):

  scriptName = "b2bTools.singleSeq.Predictor"

  def __init__(self):

    print("Initializing DynaMine...", end=' ')
    self.dynaMine = DynaMine()

    print("Initialising EFoldMine...")
    self.eFoldMine = EFoldMine()

    print("Initialising DisoMine...")
    self.disoMine = DisoMine()

    print("Initialising Agmata...")
    self.agmata = Agmata()

    print("Done.")

    # Additional info for writing files
    self.infoTexts = list(set(self.dynaMine.infoTexts + self.eFoldMine.infoTexts + self.disoMine.infoTexts + self.agmata.infoTexts))
    self.infoTexts.sort()

    self.references = list(set(self.dynaMine.references + self.eFoldMine.references + self.disoMine.references + self.agmata.references))
    self.references.sort()

    self.version = "DynaMine {}, EFoldMine {}, DisoMine {}, Agmata {}".format(self.dynaMine.version,self.eFoldMine.version,self.disoMine.version, self.agmata.version)


    self.informationPerPredictor = self.dynaMine.informationPerPredictor.copy()
    self.informationPerPredictor.update(self.eFoldMine.informationPerPredictor)
    self.informationPerPredictor.update(self.disoMine.informationPerPredictor)
    self.informationPerPredictor.update(self.agmata.informationPerPredictor)

    #self.references = ['doi: 10.1038/ncomms3741 (2013)', 'doi: 10.1093/nar/gku270 (2014)', 'doi: 10.1038/s41598-017-08366-3 (2017)']

  def predictSeqs(self, seqs, predTypes = ('eFoldMine', 'disoMine', 'agmata')):

    """
    :param seqs: A list of sequence ID and sequence pairs, e.g. ('mySeq', 'MYPEPTIDE')
    :param predTypes: DynaMine suite will be run default, then here can determine what else to run on top.
    :return: Nothing
    """

    self.seqs = seqs

    # DynaMine - always needs to be run
    self.dynaMine.predictSeqs(seqs)
    self.allPredictions = self.dynaMine.allPredictions

    if 'eFoldMine' in predTypes or 'disoMine' in predTypes:
      # EFoldMine
      self.eFoldMine.predictSeqs(seqs, dynaMinePreds=self.dynaMine.allPredictions)
      # This needs cleaning up!
      # TODO double-check that dynamine preds are not messed up!
      self.allPredictions = self.eFoldMine.allPredictions

    if 'disoMine' in predTypes:
      # DisoMine
      self.disoMine.allPredictions = self.allPredictions
      self.disoMine.predictSeqs(seqs)

      # TODO should also pull Psipred predictions if agmata requested, save time!

    if 'agmata' in predTypes:
      self.agmata.allPredictions = self.allPredictions
      self.agmata.predictSeqs(self.seqs)
